﻿/**
 * @fileOverview Payture.js is library to use in Payture paying templates
 * 3.1 - common method for AndroidPay, ApplePay
 */
'use strict';
/**
 * Main class
 *
*/
var Payture = {
	/**
	* Object, that contains default properties and flags
	*
	* @namespace
	* @static
	* @type {Object}
	* @property {string} [Type=JSON] Type of template - "Default" or "JSON"
	* @property {string} [TypeAPI=InPay] Type of API - "InPay" or "eWallet"
	* @property {string} [Action=PaySubmit] Type of action - "PaySubmit" or "AddSubmit"
	* @property {string} [Event=submit #payForm] Send form event
	* @property {string} [RemoveEvent=click #removeCard] Remove card event
	* @property {string} [TopErrorContainer=#errorTop) Container ID for errors text in JSON template
	* @property {string} [PaymentKey] Merchant Payment Key for delete Card
	* @property {boolean} [CheckNumbers=true] Check numbers in inputs
	* @property {boolean} [CheckLetters=true] Check characters in Card Holder field
	* @property {boolean} [DetectCardType=true] Detect card type - Visa or MasterCard
	* @property {boolean} [DisableButton=true] Disable button if required fields are not filled
	* @property {boolean} [GroupCardNumber=true] Change caret (desktop) or group Card Number (mobile and tablet)
	* @property {boolean} [ResizeCardNumberInput=false] Resize card number field for 19-digits number
	* @property {boolean} [HideCardList=true] Hide card list if card list is empty
	* @property {boolean} [MasterPass=false] Use MasterPass
	* @property {boolean} [MasterPassUseCVV=false] Use MasterPass
	* @property {boolean} [BINs=true] Show bank logos/names
	* @property {boolean} [BrainTree=false] Use BrainTree processing
	* @property {string}  [Language='RU'] Interface language 'RU'/'EN' 
	* @property {boolean} [CheckLuna=true] Check cardnumber by algorithm Luna
	* @property {boolean} [IsAmex=false] Use Amex cards
	* @property {boolean} [Fingerprint=false] Use fingerprint
	* @property {boolean} [ApplePay=false] Use ApplePay
	* @property {boolean} [AndroidPay=false] Use AndroidPay
	 */
	defaults : {
		Type 		: "JSON",
		TypeAPI		: "InPay",
		Action 		: "PaySubmit",
		Event 		: "submit #payForm",
		RemoveEvent : "click #removeCard",
		TopErrorContainer : "#errorTop",
		PaymentKey	: "",
		MasterPass  : false,
		MasterPassUseCVV : false,
		Kviku: false,
		Inbox		: false,
		BePaid      : false,
		redirect      : false,
		redirectKey   : "",
		// Options
		CheckNumbers : true,
		CheckLetters : true,
		DetectCardType : true,
		DisableButton : true,
		GroupCardNumber : true,
		ResizeCardNumberInput : false,
		HideCardList : true,
		BINs : true,
		BrainTree: false,
		Language: 'RU',
		CheckLuna: true,
		IsAmex: false,
		Fingerprint: false,
		FingerprintIsSend: false,
		ApplePay: false,
		AndroidPay: false,
		GooglePay: false,
		SamsungPay: false,
		serviceId: '9311432adebb44c986ec7c',
		/**
		 * @private
		 * @type {boolean}
		 */
		MasterPassRedirected : false,
		GoogleWebPay: false,
		/**
		 * Data object with jQuery set of fields elements
		 *
		 * @static
		 * @type {Object}
		 * @property {Array} Data.CardNumber
		 * @property {Object} Data.EMonth
		 * @property {Object} Data.EYear
		 * @property {Object} Data.CardHolder
		 * @property {Object} Data.SecureCode
		 */
		Data : {
			CardNumber : $("input[name=CardNumber]:visible"),
			EMonth : $("input[name=EMonth]"),
			EYear : $("input[name=EYear]"),
			CardHolder : $("input[name=CardHolder]:visible"),
			SecureCode : $("input[name=SecureCode]:visible")
		},

		/**
		 * Errors texts templates
		 *
		 * @static
		 * @type {Object}
		 * @property {string} TemplateErrors.EmptyPan
		 * @property {string} TemplateErrors.EmptyDate
		 * @property {string} TemplateErrors.EmptyMonth
		 * @property {string} TemplateErrors.EmptyYear
		 * @property {string} TemplateErrors.WrongDate
		 * @property {string} TemplateErrors.EmptyCardHolder
		 * @property {string} TemplateErrors.EmptyCVV
		 */
		TemplateErrors : templateErrors,
		/**
		 * Server errors texts
		 *
		 * @static
		 * @type {Object}
		 */
		ServerErrors : errors,
		AnswerText: answers,
		TemplateText: templateText,
		
		TimeFirstClick: 0,
		TimeSubmitStart: 0,
		TimeInit: 0,
		TotalDistance: 0,
		TimeMouseFirstClick: 0,
		fingerPrint:{},

		/**
		 * Additional function before submit form
		 * @method
		 */
		onBeforeSubmit : null,
		/**
		 * Rewrite function for change card
		 * @method
		 */
		onSelectCard : null,
		/**
		 * Rewrite function for actions after remove card
		 * @method
		 */
		onRemoveCard : null,
		/**
		 * Rewrite function for show errors
		 * @method
		 */
		onShowErrors : null,
		/**
		 * Rewrite function for hide errors
		 * @method
		 */
		onHideErrors : null,
		/**
		 * Function for success action if JSON template
		 *
		 * @param {string} redirectUrl
		 */
		onSuccess : function (redirectUrl) {
			$("form button").removeClass("loading").removeAttr("disabled");
			$(".loader").hide();
			$("#form").hide();
			$("#result").show();
			$("#result h3").text(Payture.options.AnswerText[Payture.options.Language]['PAY_SUCCESS_TITLE']);
			$("#result p:not(.message)").html(Payture.options.AnswerText[Payture.options.Language]['REDIRECT_TEXT']);
			document.title = Payture.options.TemplateText[Payture.options.Language]['RETURN_TITLE_SUCCESS'];
			$("#result a").attr("href", "#");
			if (redirectUrl) {
				$("#result a").attr("href", redirectUrl);
				setTimeout(function () { window.location.href = redirectUrl; }, 2000);
			}
		},
		/**
		 * Function for error action if JSON template
		 *
		 * @param {string} errorCode
		 * @param {boolean} canRetry
		 * @param {string} redirectUrl
		 * @param {string} key
		 */
		onError : function (errorCode, canRetry, redirectUrl, key) {
			$("form button").removeClass("loading").removeAttr("disabled");
			$(".loader").hide();

			if (!canRetry) {
				$("#form").hide();
				$("#result").show();
				$("#result h3").text(Payture.options.AnswerText[Payture.options.Language]['PAY_ERROR_TITLE']);
				$("#result .message").text(Payture.options.ServerErrors[Payture.options.Language][errorCode]);
				$("#result p:not(.message)").html(Payture.options.AnswerText[Payture.options.Language]['REDIRECT_TEXT']);
				document.title = Payture.options.TemplateText[Payture.options.Language]['RETURN_TITLE_ERROR'];
				$("#result a").attr("href", "#");
				if (redirectUrl) {
					$("#result a").attr("href", redirectUrl);
					setTimeout(function () { window.location.href = redirectUrl; }, 2000);
				}
			} else
				$(Payture.options.TopErrorContainer).text(Payture.options.ServerErrors[Payture.options.Language][errorCode]);
		},
		/**
		 * Function for success action if JSON template
		 *
		 * @param {string} redirectUrl
		 */
		onSuccessAdd : function (redirectUrl) {
			$("form button").removeClass("loading").removeAttr("disabled");
			$(".loader").hide();
			$("#form").hide();
			$("#result").show();
			$("#result h3").text(Payture.options.AnswerText[Payture.options.Language]['ADD_SUCCESS_TITLE']);
			$("#result .description").html(Payture.options.AnswerText[Payture.options.Language]['REDIRECT_TEXT']);
			$("#result a").attr("href", redirectUrl);
			setTimeout(function () { window.location.href = redirectUrl; }, 2000);
		},
		/**
		 * Function for error action if JSON template
		 *
		 * @param {string} errorCode
		 * @param {boolean} canRetry
		 * @param {string} redirectUrl
		 * @param {string} key
		 */
		onErrorAdd : function (errorCode, canRetry, redirectUrl, key) {
			$("form button").removeClass("loading").removeAttr("disabled");
			$(".loader").hide();

			if (!canRetry) {
				$("#form").hide();
				$("#result").show();
				$("#result h3").text(Payture.options.AnswerText[Payture.options.Language]['ADD_ERROR_TITLE']);
				$("#result .message").text(Payture.options.ServerErrors[Payture.options.Language][errorCode]);
				$("#result .description").html(Payture.options.AnswerText[Payture.options.Language]['REDIRECT_TEXT']);
				$("#result a").attr("href", redirectUrl);
				setTimeout(function () { window.location.href = redirectUrl; }, 2000);
			} else
				$(Payture.options.TopErrorContainer).text(Payture.options.ServerErrors[Payture.options.Language][errorCode]);
		}
	},
	/**
	 * Object, that contains mapping between english and russian letters
	 *
	 * @static
	 * @type {Object}
	 */
	enToRu : {
		'й' : 'q', 'ц' : 'w', 'у' : 'e', 'к' : 'r', 'е' : 't', 'н' : 'y', 'г' : 'u', 'ш' : 'i', 'щ' : 'o', 'з' : 'p',
		'ф' : 'a', 'ы' : 's', 'в' : 'd', 'а' : 'f', 'п' : 'g', 'р' : 'h', 'о' : 'j', 'л' : 'k', 'д' : 'l',
		'я' : 'z', 'ч' : 'x', 'с' : 'c', 'м' : 'v', 'и' : 'b', 'т' : 'n', 'ь' : 'm'
	},
	/**
	 * Object, that contains test with common regexps
	 *
	 * @namespace
	 * @static
	 * @type {Object}
	 */
	regexp : {
		/**
		 * @param {Number|String} cCode
		 * @returns {boolean}
		 */
		isNumber : function (cCode) { return (/[0-9]/.test(String.fromCharCode(cCode))); },
		isAmex : function (cCode, string) { if(string == '') return /(3)/.test(String.fromCharCode(cCode)); else return /[0-9]/.test(String.fromCharCode(cCode)); },
		isNotAmex : function (cCode, string) { if(string == '') return /(1|2|4|5|6|7|8)/.test(String.fromCharCode(cCode)); else return /[0-9]/.test(String.fromCharCode(cCode)); },
		/**
		 * @param {Number|String} cCode
		 * @returns {boolean}
		 */
		isPan : function  (cCode) { return /3|4|5|6/.test(String.fromCharCode(cCode)); }
	},
	/**
	 * Empty object for filling with options after initialization
	 *
	 * @static
	 * @type {Object}
	 */
	options : {},
	/**
	 * @typedef {Object} FormValues
	 * @property {String} [CardNumber]
	 * @property {Number} [EMonth]
	 * @property {Number} [EYear]
	 * @property {String} [CardHolder]
	 * @property {Number} [SecureCode]
	 * @property {String} [CardId] - when eWallet used
	 * @property {Boolean} [AddCard] - when eWallet used
	 * @property {String} [oauth_Token] - when MasterPass used
	 */
	
	apiPath: function() {
		switch (Payture.options.TypeAPI) {
			case "eWallet":
				return "/vwapi";
			default:
			case "InPay": 
				return "/apim";
		}
	},
	/**
	 * Pay in InPay Interface
	 *
	 * @param {Object} params - Custom options
	 */
	InPay : function (params) {
		Payture.commonInit(params, "InPay", {
			CardNumber : $("input[name='CardNumber']:visible"),
			EMonth : $("input[name='EMonth']"),
			EYear : $("input[name='EYear']"),
			EDate : $("input[name='DateTo']"),
			CardHolder : $("input[name='CardHolder']:visible"),
			SecureCode : $("input[name='SecureCode']:visible")
		});
		$('.addCard, .selectCard').hide();
		$('.payBlock button').text(Payture.options.TemplateText[Payture.options.Language].PAYCARD);
		$("html " + Payture.options.Event.split(" ")[1]).bind(Payture.options.Event.split(" ")[0], function () {
			if (Payture.options.onBeforeSubmit != null)
				Payture.options.onBeforeSubmit();

			$("form button").addClass("loading").attr("disabled", true).prop('disabled', true);
			if(Payture.options.MasterPass && Payture.options.MasterPassRedirected){
				Payture.Pay({
					Type: "MP",
					Key: $("input[name='Key']").val(),
					Card: Payture.getFormValues({
						SecureCode : $("input[name='SecureCode']:visible"),
						oauth_Token: $("input[name='oauth_Token']")
					})
				});
			} else {
				Payture.Pay({
					Type : Payture.options.Type,
					TypeAPI : "InPay",
					Key : $("input[name='Key']").val(),
					Card : Payture.getFormValues(Payture.options.Data)
				});
			}
			return false;
		});
		if(Payture.options.ApplePay) Payture.ApplePayInit();
		if(Payture.options.AndroidPay) Payture.AndroidPayInit();
		if(Payture.options.GooglePay) Payture.GooglePayInit();
		if(Payture.options.GoogleWebPay) Payture.GoogleWebPayInit();
		if(Payture.options.SamsungPay) Payture.SamsungPayInit();
	},
	/**
	 * Pay in eWallet Interface
	 *
	 * @param {Object} params - custom options
	 */
	eWalletPay : function (params) {
		Payture.commonInit(params, "eWallet", {
			CardId : $("select[name='CardId']"),
			CardNumber : $("input[name='CardNumber']:visible"),
			EMonth : $("input[name='EMonth']"),
			EYear : $("input[name='EYear']"),
			EDate : $("input[name='DateTo']"),
			CardHolder : $("input[name='CardHolder']:visible"),
			SecureCode : $("input[name='SecureCode']:visible"),
			AddCard : $("input[name='AddCard']")
		});
		$('.payBlock button').text(Payture.options.TemplateText[Payture.options.Language].PAYCARD);
		Payture.changeCard($(Payture.options.Data.CardId).val(), $(Payture.options.Data.CardId).find("option:selected").text());
        $(Payture.options.Data.CardId).change(function() {
            Payture.changeCard($(this).val(), $(this).find("option:selected").text())
        });
		$("html " + Payture.options.Event.split(" ")[1]).bind(Payture.options.Event.split(" ")[0], function () {
			if (Payture.options.onBeforeSubmit != null) {
				Payture.options.onBeforeSubmit();
			}

			$("form button").addClass("loading").attr("disabled", true).prop('disabled', true);
			if(Payture.options.MasterPass && Payture.options.MasterPassRedirected){
				Payture.Pay({
					Type: "MP",
					Key: $("input[name='Key']").val(),
					Card: Payture.getFormValues({
						CardId : $("select[name='CardId']:visible"),
						SecureCode : $("input[name='SecureCode']:visible"),
						oauth_Token: $("input[name='oauth_Token']"),
						AddCard : $("input[name='AddCard']:visible")
					})
				});
			} else {
				Payture.Pay({
					Type: Payture.options.Type,
					TypeAPI: "eWallet",
					Key: $("input[name='Key']").val(),
					Card: Payture.getFormValues(Payture.options.Data)
				});
			}
			return false;
		});

		$("html " + Payture.options.RemoveEvent.split(" ")[1]).bind(Payture.options.RemoveEvent.split(" ")[0], function () {
			if ($(Payture.options.Data.CardId).val() != "FreePay" && $(Payture.options.Data.CardId).val() != "")
				Payture.RemoveCardBySession({ CardId : $(Payture.options.Data.CardId).val() })
			return false;
		});
		if(Payture.options.ApplePay) Payture.ApplePayInit();
		if(Payture.options.AndroidPay) Payture.AndroidPayInit();
		if(Payture.options.GooglePay) Payture.GooglePayInit();
		if(Payture.options.GoogleWebPay) Payture.GoogleWebPayInit();
		if(Payture.options.SamsungPay) Payture.SamsungPayInit();
	},
	/**
	 * Add card in eWallet Interface
	 *
	 * @param {Object} params - custom options
	 */
	eWalletAdd : function (params) {
		Payture.commonInit(params, "eWallet", {
			CardNumber : $("input[name='CardNumber']:visible"),
			EMonth : $("input[name='EMonth']"),
			EYear : $("input[name='EYear']"),
			EDate : $("input[name='DateTo']"),
			CardHolder : $("input[name='CardHolder']:visible"),
			SecureCode : $("input[name='SecureCode']:visible")
		});
		$('.payBlock button').text(Payture.options.TemplateText[Payture.options.Language].ADDCARD);
		$('.payBlock .title').text(Payture.options.TemplateText[Payture.options.Language].PAYFORMTITLEADD);
		$("html " + Payture.options.Event.split(" ")[1]).bind(Payture.options.Event.split(" ")[0], function () {
			if (Payture.options.onBeforeSubmit != null)
				Payture.options.onBeforeSubmit();

			$("form button").addClass("loading").attr("disabled", true).prop('disabled', true);
			Payture.Add({
				Type : Payture.options.Type,
				Key : $("input[name='Key']").val(),
				Card : Payture.getFormValues(Payture.options.Data)
			});
			return false;
		});
	},
	/**
	 * Pay card in Inpay and eWallet interfaces
	 *
	 * @param {Object} params
	 * @param {string} params.Type
	 * @param {string} [params.TypeAPI]
	 * @param {string} params.Key
	 * @param {FormValues} params.Card
	 * @param {function} [params.onSuccess]
	 * @param {function} [params.onError]
	 */
	Pay : function (params) {
		if (!params.onSuccess && params.Type == "JSON")
			params.onSuccess = Payture.options.onSuccess;
		if (!params.onError && params.Type == "JSON")
			params.onError = Payture.options.onError;
		
		
		if(!Payture.options.BrainTree || ($(Payture.options.Data.CardId).length > 0 && $(Payture.options.Data.CardId).val() != "FreePay" && $(Payture.options.Data.CardId).val() != ""))
		{
			if (params.Type == "Default") Payture.submitForm("PaySubmit", Payture.inlineData(params.Key, params.Card));
			else if (params.Type == "MP") Payture.submitForm("PaySubmitMP", Payture.inlineData(params.Key, params.Card));
			else if (params.Type == "JSON") Payture.submitFormJSON("PaySubmit", params.TypeAPI, Payture.inlineData(params.Key, params.Card), params.onSuccess, params.onError);
						
		}
		else
		{
			Payture.brainTreePay(params.Card, params);				
		}
	},
	/**
	 * Add card in eWallet interface
	 *
	 * @param {Object} params
	 * @param {string} params.Type
	 * @param {string} params.Key
	 * @param {FormValues} params.Card
	 * @param {function} [params.onSuccess]
	 * @param {function} [params.onError]
	 */
	Add : function (params) {
		if (!params.onSuccess && params.Type == "JSON")
			params.onSuccess = Payture.options.onSuccessAdd;
		if (!params.onError && params.Type == "JSON")
			params.onError = Payture.options.onErrorAdd;		
		
		if(!Payture.options.BrainTree || ($(Payture.options.Data.CardId).length > 0 && $(Payture.options.Data.CardId).val() != "FreePay" && $(Payture.options.Data.CardId).val() != ""))
		{
			if (params.Type == "Default") Payture.submitForm("AddSubmit", Payture.inlineData(params.Key, params.Card));
			else if (params.Type == "JSON") Payture.submitFormJSON("AddSubmit", "eWallet", Payture.inlineData(params.Key, params.Card), params.onSuccess, params.onError);
		}
		else
		{
			Payture.brainTreeAdd(params.Card, params);				
		}		
		
	},
	/**
	 * Validate field value
	 *
	 * @param {Object} params
	 * @param {string} params.Name
	 * @param {string} params.Value
	 * @return {validationData|null}
	 */
	Validate : function (params) { return Payture.validationData[params.Name] ? Payture.validationData[params.Name](params.Value) : null; },
	/**
	 * Get extended card list
	 *
	 * @return {CardList} Card list
	 */
	GetCardList : function () {
		var CardList = {};
		if (CardPANS && CardPANS.length != 0) {
			$.each(CardPANS, function (index, value) {
				/**
				 * @typedef {Object} CardList
				 * @property {number} Id
				 * @property {boolean} NoCVV
				 * @property {boolean} Expired
				 * @property {Date} LastPay
				 */
				CardList[value] = {
					Id : index,
					NoCVV : CardNoCVVInfo && CardNoCVVInfo[index] ? CardNoCVVInfo[index] : undefined,
					Expired : CardExpiredInfo && CardExpiredInfo[index] ? CardExpiredInfo[index] : undefined,
					LastPay : CardLastSuccess && CardLastSuccess[index] ? new Date(CardLastSuccess[index]) : undefined
				}
			});
		}
		return CardList;
	},
	/**
	 * Validate field value
	 *
	 * @param {Object} params
	 * @param {string} params.CardId
	 * @param {string} params.PaymentKey
	 * @param {string} [params.onRemove]
	 */
	RemoveCard : function (params) {
		$.ajax({
			type : "POST",
			url : "/vwapi/Remove2",
			data:{ 'CardId': params.CardId, 'VWID': params.PaymentKey },
			timeout : 60000,
			success : function (data) {
				var response = Payture.xmlToString(data);
				if (response.indexOf('Success="True"') != -1) {
					if (params.onRemove && params.onRemove !== null) params.onRemove(params.CardId);
					else Payture.updateList(params.CardId);
				} 
			},
			error : function () {
				alert("Произошла ошибка");
			}
		});
	},
	RemoveCardBySession : function (params) {
		var answer = confirm(Payture.options.TemplateText[Payture.options.Language].DELETEALERT);
		
		if(answer)
		{
			$.ajax({
				type : "POST",
				url : "/vwapi/RemoveBySession",
				data:{ 'CardId': params.CardId, 'SessionId': QueryString.SessionId },
				timeout : 60000,
				success : function (data) {
					var response = Payture.xmlToString(data);
					if (response.indexOf('Success="True"') != -1) {
						if (params.onRemove && params.onRemove !== null) params.onRemove(params.CardId);
						else Payture.updateList(params.CardId);
						Payture.checkButton();
					} 
				},
				error : function () {
					alert("Произошла ошибка");
				}
			});
		}
		
	},
	/**
	 * Validate field value
	 *
	 * @param {String} key
	 * @param {Int} paymentType
	 * @param {Function} callbackSuccess
	 * @param {Function} callbackError
	 */
	GetState : function (key, paymentType, callbackSuccess, callbackError) {
		Payture.Timeout += 30000;
		$.ajax({
			type : "POST",
			url : "/ncapi/GetState",
			data : {
				Key : key,
				NoCardPaymentType : paymentType,
				CompileRedirectURL : true
			},
			timeout : 60000,
			success : function (data) {
				if (data.Success && data.State == "Charged") {
					if (callbackSuccess)
						callbackSuccess(data.ReturnUrl);
				} else if (data.Success && data.State == "Authorized" && Payture.Timeout < 600000) {
					setTimeout(function () {
						Payture.GetState(key, paymentType, callbackSuccess, callbackError);
					}, 30000);
				} else {
					if (callbackError)
						callbackError(data.ErrCode, false, data.ReturnUrl)
				}
				
			},
			error : function () {
				alert("Произошла ошибка");
			}
		});
	},
	translatePageText : function(){
		$('input[name=CardHolder]').attr('placeholder',Payture.options.TemplateText[Payture.options.Language].CARDHOLDERP);
		if(Payture.options.IsAmex){
			$('input[name=CardNumber]').attr('placeholder',Payture.options.TemplateText[Payture.options.Language].CARDNUMBERP_AMEX);		
		}
		else{
			$('input[name=CardNumber]').attr('placeholder',Payture.options.TemplateText[Payture.options.Language].CARDNUMBERP);			
		}
		var payFormTitle = $('#payForm .title').text();
		payFormTitle = payFormTitle.replace('{PAYFORMTITLE1}', Payture.options.TemplateText[Payture.options.Language].PAYFORMTITLE1);
		payFormTitle = payFormTitle.replace('{PAYFORMTITLE2}', Payture.options.TemplateText[Payture.options.Language].PAYFORMTITLE2);
		payFormTitle = payFormTitle.replace('{PAYFORMTITLE3}', Payture.options.TemplateText[Payture.options.Language].PAYFORMTITLE3);
		
		$('#payForm .title').text(payFormTitle);
		$('#apple-pay-button .text').text(Payture.options.TemplateText[Payture.options.Language].PAYWITH);
		$('.container-kviku button').text(Payture.options.TemplateText[Payture.options.Language].KVIKUPAY);
		$('#google-pay-button').text(Payture.options.TemplateText[Payture.options.Language].PAYWITHSAVEDCARD);

		
		$('input[name=EMonth]').attr('placeholder',Payture.options.TemplateText[Payture.options.Language].MONTHP);
		$('input[name=EYear]').attr('placeholder',Payture.options.TemplateText[Payture.options.Language].YEARP);		
		
		$('input[name=Phone]').attr('placeholder',Payture.options.TemplateText[Payture.options.Language].PHONE);
		
		$('.ExpiredDate .label').html(Payture.options.TemplateText[Payture.options.Language].VALID);
		$('.CardNumber .label .bankName').text(Payture.options.TemplateText[Payture.options.Language].CARDNUMBER);
		$('.CardHolder .label').text(Payture.options.TemplateText[Payture.options.Language].CARDHOLDER);
		$('.comment').text(Payture.options.TemplateText[Payture.options.Language].NEEDEDFIELDS);
		$('footer .text').html(Payture.options.TemplateText[Payture.options.Language].SECURITYTEXT);
		
		$('#bank-card-one').attr('placeholder', Payture.options.TemplateText[Payture.options.Language].WELCOME);
		$('.container-masterpass .more a').text(Payture.options.TemplateText[Payture.options.Language].MORE);
		$('#gift').text(Payture.options.TemplateText[Payture.options.Language].GIFT);
		$('#return-description').html(Payture.options.TemplateText[Payture.options.Language].RETURNTEXT);
		$('.month').html(Payture.options.TemplateText[Payture.options.Language].MONTH);
		$('.month').next('.label').html(Payture.options.TemplateText[Payture.options.Language].YEAR);
		$('.valid').html(Payture.options.TemplateText[Payture.options.Language].VALID);
		$('#removeCard').html(Payture.options.TemplateText[Payture.options.Language].DELETECARD);
		$('#addCard').contents().filter(function() {return this.nodeType == 3}).replaceWith(Payture.options.TemplateText[Payture.options.Language].ADDCHECKBOX)	
		$('.selectCard span').html(Payture.options.TemplateText[Payture.options.Language].SELECTCARD);
		$('[name=CardId] option').eq(0).text(Payture.options.TemplateText[Payture.options.Language].DEFAULTCARD);
		//$('#paymentCard h2').html(Payture.options.TemplateText[Payture.options.Language].PAYMENTDESCRIPTION);
		if(Payture.options.IsAmex){
			$('.CVV .numbers').html('1234');
			$('.CVV .info').html(Payture.options.TemplateText[Payture.options.Language].UNDERCVVAMEX);
			$('[name=SecureCode]').attr('maxlength',4).attr('placeholder','_ _ _ _');
		}
		else{
			$('.CVV .numbers').html('123');
			$('.CVV .info').html(Payture.options.TemplateText[Payture.options.Language].UNDERCVV);
			$('[name=SecureCode]').attr('maxlength',3).attr('placeholder','_ _ _');
		}
		if(Payture.options.MasterPass) $('.container-masterpass').show();
		if(Payture.options.Kviku) $('.container-kviku').show();
	},
	/**
	 * Common initialize for all methods
	 *
	 * @param {Object} params - init params
	 * @param {String} typeAPI
	 * @param {Object} data - default data params
	 */
	commonInit : function (params, typeAPI, data) {
	 	var dstart = new Date();
		if (!params)
			params = {};
		Payture.options = $.extend({}, Payture.defaults, params);
		Payture.options.TypeAPI = typeAPI;
		Payture.options.TimeInit = dstart.getTime()
			
		$('input[name=CardNumber]').payment('formatCardNumber');
		$("#date").mask("99/99");
		$('input[name=Phone]').mask('(999) 999-9999');			
			
		if(Payture.options.MasterPass){	
			$('body').delegate('.more a:visible', 'click', function(){
				$( ".more-content" ).css({
					'width': '582px',
					'top':'30px',
					'left': ($(window).width() - 582)/2 + 'px',
					'position':'fixed',
					'zIndex':'100'
				}).show();
				$( ".more-content" ).append('<div class="close">X</div>');
				$('.close').css({'position':'absolute','top':'24px','right':'50px'});
				$('body').append('<div id="cover"></div>');
				$('#cover').css({
					'position':'fixed',
					'top':'0',
					'width':'100%',
					'height':'100%',
					'opacity':'0.6',
					'backgroundColor':'#000000',
					'zIndex':'99'})
				.show();
			});
		    $('body').delegate('.more-content .close','click',function() { 
				$('.more-content').hide(); 
				$('#cover').remove();
				$('.more-content .close').remove();
			});	
		}
		var lastSeenAt = {x: null, y: null};
		$(document).mousemove(function(event) {
			if(lastSeenAt.x) {
				Payture.options.TotalDistance += Math.sqrt(Math.pow(lastSeenAt.y - event.clientY, 2) + Math.pow(lastSeenAt.x - event.clientX, 2));
				
							 }
			lastSeenAt.x = event.clientX;
			lastSeenAt.y = event.clientY;
										});
		$(document).keydown( function(event) {
			
			if(Payture.options.TimeMouseFirstClick == 0)
			{
				var d = new Date();
				Payture.options.TimeMouseFirstClick = d.getTime() - Payture.options.TimeInit;
			}
			
		});
		$(document).click( function(event) {
			
			if(Payture.options.TimeFirstClick == 0)
			{
				var d = new Date();
				Payture.options.TimeFirstClick = d.getTime() - Payture.options.TimeInit;
			}
			
		});
		$(document).keypress( function(event) {
			
			if(Payture.options.TimeFirstClick == 0)
			{
				var d = new Date();
				Payture.options.TimeFirstClick = d.getTime() - Payture.options.TimeInit;
			}
			
		});
		if (!params.Data)
			Payture.options.Data = data;

		if (params.TemplateErrors)
			Payture.options.TemplateErrors[Payture.options.Language] = $.extend(Payture.defaults.TemplateErrors[Payture.options.Language], params.TemplateErrors);

		if (params.ServerErrors)
			Payture.options.ServerErrors[Payture.options.Language] = $.extend(Payture.defaults.TemplateErrors[Payture.options.Language], params.ServerErrors);

		Payture.translatePageText();
		
		if(Payture.options.IsAmex){
			Payture.options.TemplateErrors['EN'].EmptyPan = "Input 15 digits of the card number";
			Payture.options.TemplateErrors['RU'].EmptyPan = "Введите 15 знаков номера карты";
		}
		
		$(".section").hide();
		if (Payture.options.Inbox) {
			$("#paymentType").show();
			Payture.inboxActions();
		}
		else if(Payture.options.BePaid) 
		{
			Payture.bepaidActions();
		}
		else if(Payture.options.redirect) 
		{
			Payture.redirectActions();
		}
		else $("#paymentCard").show();

		
		
		if (Payture.options.Type == "JSON" && !$.isEmptyObject(Payture.getJSONResponse())) {
			Payture.options.onError(
				Payture.getJSONResponse().ErrCode,
				Payture.getJSONResponse().CanRetry,
				Payture.getJSONResponse().RedirectUrl
			);
		}

		Payture.formActions();
		
		if(Payture.options.Fingerprint)
		{
			$.extend(Payture.options.Data,{PaytureId : $("#PaytureId")});
			var fp = new Fingerprint2();
			fp.get(function (result, keys) {
				if (typeof window.console !== "undefined") {
					keys.push({
						key:'fingerprint',
						value: result
					});
				}
				Payture.options.fingerPrint = keys;
			});				
		}


		if (Payture.options.MasterPass)
			Payture.MasterPass();
	},
	/** 
	 * Update Card List after removing card
	 *
	 * @param {string} cardId - Removed Card Id
	 */
	updateList : function (cardId) {
		if (Payture.options.onRemoveCard !== null)
			Payture.options.onRemoveCard(cardId);
		else {
			$(Payture.options.Data.CardId).find("option[value=" + cardId + "]").remove();
			if (Payture.options.HideCardList && Payture.options.TypeAPI == "eWallet")
				Payture.hideCardList();
			
			Payture.changeCard($(Payture.options.Data.CardId).val(), $(Payture.options.Data.CardId).find("option:selected").text());
		}
	},
	/** 
	 * Generate links for different payment types
	 */
	getPayLinks : function () {
		var data = "Key=" + $("input[name=Key]").val() + ";isNoCard=true;NoCardPaymentType=",
			location = window.location;
		$("#paymentType a:not(.payCard)").each(function () {
			$(this).attr("href", location.protocol + "//" + location.host + "/apim/PaySubmit/?Data=" + encodeURIComponent(data + $(this).attr("rel")));
		});
	},
	/**
	 * Set new key after JSON error
	 *
	 * @param {String} key
	 */
	setNewKey : function (key) { $("input[name=Key]").val(key); },
	/**
	 * Parse JSON resoponse on templates
	 * @return {Object} response
	 */
	getJSONResponse : function () {
		var responseJSON = $(".response_json").text();
		return responseJSON.indexOf('{response_json}') == -1 && responseJSON != "" ? eval ('(' + responseJSON + ')') : {};
	},
	inboxActions : function () {
		Payture.getPayLinks();
		$("#paymentType a").click(function () {
			if ($(this).is(".payCard")) {
				$(".section").hide();
				$("#paymentCard").show();
				return false;
			} else {
				$(".loader").fadeIn(150);
				Payture.NoCardPaymentType = $(this).attr("rel");
				Payture.Timeout = 0;
				setTimeout(function () {
					Payture.GetState($("input[name=Key]").val(), Payture.NoCardPaymentType, Payture.options.onSuccess, Payture.options.onError);
				}, 30000);
			}
		});

		$("#reset").click(function () {
			$(".loader").hide();
			Payture.NoCardPaymentType = undefined;
			Payture.Timeout = 0;
		});

		$("#toBack").click(function () {
			$(".section").hide();
			$("#paymentType").show();
			return false;
		});
	},
	bepaidActions : function () {
		var data = "Key=" + $("input[name=Key]").val() + ";isNoCard=true;NoCardPaymentType=BePaid";
		$.ajax({
			type : "POST",
			url : Payture.apiPath() + "/PaySubmit",
			data : {
				Data : data,
				Json : true
			},
			timeout : 60000,
			success : function (data) {
				window.location.href = data.RedirectUrl;
			},
			error : function () {
				alert(Payture.options.TemplateText[Payture.options.Language].ERRORALERT);
			}
		});		
	},
	kvikuActions : function () {
		var data = "Key=" + $("input[name=Key]").val() + ";isNoCard=true;NoCardPaymentType=Kviku";
		$.ajax({
			type : "POST",
			url : Payture.apiPath() + "/PaySubmit",
			data : {
				Data : data,
				Json : true
			},
			timeout : 60000,
			success : function (data) {
				window.location.href = data.RedirectUrl;
			},
			error : function () {
				alert(Payture.options.TemplateText[Payture.options.Language].ERRORALERT);
			}
		});		
	},
	redirectActions: function () {
		var data = "Key=" + $("input[name=Key]").val() + ";isNoCard=true;NoCardPaymentType=" + Payture.options.redirectKey;
		$.ajax({
			type : "POST",
			url : Payture.apiPath() + "/PaySubmit",
			data : {
				Data : data,
				Json : true
			},
			timeout : 60000,
			success : function (data) {
				if(data.RedirectUrl)
					window.location.href = data.RedirectUrl;
				else
					alert(Payture.options.TemplateText[Payture.options.Language].ERRORALERT);
			},
			error : function () {
				alert(Payture.options.TemplateText[Payture.options.Language].ERRORALERT);
			}
		});		
	},
	getSelectionText:function(){
		var txt = '';
		if (txt = window.getSelection) {
			txt = window.getSelection();
		} else { 
			txt = document.selection.createRange();
		}
		return txt;
	},
	removeSubstring:function(str, startIndex, count) {
		return str.substr(0, startIndex) + str.substr(startIndex + count);
	},
	/**
	 * Actions with inputs:<br/>
	 * &nbsp;   Disable buttons<br/>
	 * &nbsp;   Select card type<br/>
	 * &nbsp;   Resize card number input<br/>
	 * &nbsp;   Validate charapters on KEYPRESS<br/>
	 * &nbsp;   Change caret position or grouping number on KEYUP<br/>
	 * &nbsp;   Validate values end show errors on BLUR<br/>
	 */
	formActions : function () {
		if (Payture.options.DisableButton)
			Payture.checkButton();
		if (Payture.options.DetectCardType)
			Payture.selectCardType(Payture.getPAN(Payture.options.Data.CardNumber).replace(/ /g,''));
		if (Payture.options.HideCardList && Payture.options.TypeAPI == "eWallet")
			Payture.hideCardList();

		var focusElement = "";
		$("*").focus(function () {
			focusElement = Payture.defineFieldType(this);
		});


		$("html input, select").bind({
			click: function(event){
				if($(this).is("select")) 	Payture.hideErrors($(this));
			},
			keypress : function (event) {
				if ((event.which == 118 && event.metaKey) || (event.which == 118 && event.ctrlKey) || (event.which == 17 && event.key == 'Control' )){
					
				} else {
					var charCode = (event.which) ? event.which : event.keyCode;
					if(charCode != 8 && charCode != 9 && charCode != 37 && charCode != 38 && charCode != 39 && charCode != 40){
						Payture.hideErrors($(this));
						
						
						var start =	$(this)[0].selectionStart;
						var end =	$(this)[0].selectionEnd;
						
						var diff = Math.abs(end - start);
						
						if(diff > 0)
						{
							$(this).val(Payture.removeSubstring($(this).val(),start,diff));
						}
						
						if(!Payture.options.IsAmex){
							if ($(this).is(".onlyCardNum") && Payture.options.CheckNumbers) return Payture.checkString($(this).val(), event, "isNotAmex");
						}
						else{
							if ($(this).is(".onlyCardNum") && Payture.options.CheckNumbers) return Payture.checkString($(this).val(), event, "isAmex");
						}				
						
						if ($(this).is(".onlyNum") && Payture.options.CheckNumbers) return Payture.checkChar(event, "isNumber");
						if ($(this).is(".onlyLat") && Payture.options.CheckLetters) return Payture.translateText(event, this);
					}
				}
			},
			paste: function(event){
				var _this = this;

				if (Payture.options.BINs && Payture.defineFieldType($(event.currentTarget)) == "CardNumber")
			        setTimeout( function() {
						var text = $(_this).val();
						Payture.checkBinsOnPaste('card-one',"CardNumber",text);
						//Payture.checkBins('card-one',"CardNumber");
					}, 100);													
			},
			keyup : function (event) {	
				if (Payture.options.DetectCardType && Payture.defineFieldType($(event.currentTarget)) == "CardNumber")
					Payture.selectCardType(Payture.getPAN(Payture.options.Data.CardNumber).replace(/ /g,''));
				if (Payture.options.DisableButton)
					Payture.checkButton();

				if (Payture.options.BINs && Payture.defineFieldType($(event.currentTarget)) == "CardNumber")
					Payture.checkBins('card-one',"CardNumber");
				
				if (Payture.options.GroupCardNumber) {
					var element = Payture.defineFieldType($(event.currentTarget));
					if (element == "CardNumber" && !Array.isArray(element))
						Payture.separatingNumber(event, this);
					
					Payture.changeCaretPosition(event, this);
				}
			},
			blur : function (event) {			
				setTimeout(function() {
					if (Payture.defineFieldType($(event.currentTarget)) != focusElement) {
						Payture.showErrors(Payture.Validate({
							Name : Payture.defineFieldType($(event.currentTarget)), 
							Value : Payture.getFormValues(Payture.options.Data)[Payture.defineFieldType($(event.currentTarget))]
						}));
					}
				}, 10);
			}
		});
	},
	checkBinsOnPaste : function(elementId,fieldName,numberText) {
		var number = numberText.replace(/ /g,'');
		var breaker = false;
		if(number.length >= 6)
		{
			number = number.substring(0,6);
			for(var i = 0; i < bins.length; i++)
			{
				if($.inArray(parseInt(number),bins[i].BINS) != -1)
					break;
				breaker = false;
				for(var j = 0; j < bins[i].BINS.length; j++)
				{
					if(parseInt(number).toString().indexOf(bins[i].BINS[j].toString()) != -1)
					{
						breaker = true;
						break;
					}
				}
				if(breaker) break;
			}
			
			if(i < bins.length)
			{
				if(bins[i].Color && bins[i].Color != '')
				{
					$('#' + elementId).css('backgroundColor','#'+bins[i].Color);
				}
				else
				{
					$('#' + elementId).css('backgroundColor','#ffffff');
				}
				if(bins[i].Name && bins[i].Name != '')
				{			
					$('#bank-' + elementId).val(bins[i].Name);
					$('#' + elementId + ' .holder.inactive').addClass('filled');
				}
				else
				{
					$('#bank-' + elementId).val('');
					$('#' + elementId + ' .holder.inactive').removeClass('filled');
				}
			}
			else
			{
				if(elementId == 'card-one')
				{
					$('#' + elementId).css('backgroundColor','#ffffff');
					$('#' + elementId + ' .holder.inactive').removeClass('filled');
					$('#bank-' + elementId).val('');
				}

	
			}
		}
		else if(number.length < 6)
		{
			if(elementId == 'card-one')
			{
				$('#' + elementId).css('backgroundColor','#ffffff');
			    $('#' + elementId + ' .holder.inactive').removeClass('filled');
				$('#bank-' + elementId).val('');
			}
			
		}
	},
	checkBins : function(elementId,fieldName) {
		var number = Payture.getPAN(Payture.options.Data[fieldName]).replace(/ /g,'');
		var breaker = false;
		if(number.length >= 6)
		{
			number = number.substring(0,6);
			for(var i = 0; i < bins.length; i++)
			{
				if($.inArray(parseInt(number),bins[i].BINS) != -1)
					break;
				breaker = false;
				for(var j = 0; j < bins[i].BINS.length; j++)
				{
					if(parseInt(number).toString().indexOf(bins[i].BINS[j].toString()) != -1)
					{
						breaker = true;
						break;
					}
				}
				if(breaker) break;
			}
			
			if(i < bins.length)
			{
				if(bins[i].Color && bins[i].Color != '')
				{
					$('#' + elementId).css('backgroundColor','#'+bins[i].Color);				
				}
				else
				{
					$('#' + elementId).css('backgroundColor','#ffffff');
				}
				if(bins[i].Name && bins[i].Name != '')
				{			
					$('#bank-' + elementId).val(bins[i].Name);
					$('#' + elementId + ' .holder.inactive').addClass('filled');
				}
				else
				{
					$('#bank-' + elementId).val('');
					$('#' + elementId + ' .holder.inactive').removeClass('filled');
				}
			}
			else
			{
				if(elementId == 'card-one')
				{
					$('#' + elementId).css('backgroundColor','#ffffff');
					$('#' + elementId + ' .holder.inactive').removeClass('filled');
					$('#bank-' + elementId).val('');
				}
			}
		}
		else if(number.length < 6)
		{
			if(elementId == 'card-one')
			{
				$('#' + elementId).css('backgroundColor','#ffffff');
			    $('#' + elementId + ' .holder.inactive').removeClass('filled');
				$('#bank-' + elementId).val('');
			}
			
		
		}
	},
	/**
	 * Hide card list if no options
	 */
	hideCardList : function () {
		if ($(Payture.options.Data.CardId).length > 0 && $(Payture.options.Data.CardId).find("option").length < 2)
			$(".selectCard").hide();
		else
			$(".selectCard").show();
	},
	/**
	 * Set card parameters on change card
	 *
	 * @param {string} cardId
	 * @param {string} cardMask
	 */
	changeCard : function (cardId, cardMask) {
		if (Payture.options.onSelectCard !== null)
			Payture.options.onSelectCard(cardId, cardMask);
		else {
			if (cardId == "FreePay" || typeof cardId == 'undefined') {
				$.each(Payture.options.Data, function (index, value) {
					if (Array.isArray(value)) {
						$.each(value, function (i, v) {
							$(v).val("").removeAttr("disabled");
						})
					} else if ($(value).is("input") && !$(value).hasClass('winpay')) {
						$(value).val("").removeAttr("disabled");
					}
				});
				$("#addCard").show().find("input").removeAttr("disabled");
				$("html " + Payture.options.RemoveEvent.split(" ")[1]).hide();
			} else {
				Payture.showCardStub(cardMask, true);
				$("#addCard").hide().find("input").attr("disabled", true);
				$("html " + Payture.options.RemoveEvent.split(" ")[1]).show();
				Payture.selectCardType(cardMask.substring(0,6));
				$('input[name=CardNumber]').keyup();
			}
			$.each(Payture.options.Data, function (index, value) { Payture.hideErrors(value); });
		}
		Payture.checkBins('card-one',"CardNumber");
		Payture.checkButton();
	},
	/**
	 * Get form values
	 *
	 * @param {Object} data - object of elements
	 * @return {FormValues} values
	 */
	getFormValues : function (data) {
		var values = {};
		$.each(data, function (index, value) {
			if (Array.isArray(value)) {
				if ($(value[0]).is(":disabled"))
					return true;
				values[index] = Payture.getPAN(value).replace(/ /g,'');
			} else if ($(value).is(":input")) {
				if ($(value).is(":disabled") && index != "DESTINATION" && index != "CLIENT")
					return true;
				values[index] = $(value).val();
				if (index == "EMonth") values[index] = Payture.addValue($(value).val());
				if (index == "CardNumber") values[index] = $(value).val().replace(/\s/g, "");
				if (index == "CLIENT") values[index] = $(value).val().replace(/\s/g, "");
				if (index == "AddCard") values[index] = $(value).is(":checked") ? "True" : "";
				if (index == "PHONE") values[index] = $(value).val().replace(/(\s|\)|\(|-)/g, "");				
			} else if (typeof(value) == "function" || typeof(value) == "string") {
				values[index] = value;
			}
		});
		return values;
	},
	/**
	 * Define field name
	 *
	 * @param {HTMLElement} element
	 * @return {String} name
	 */
	defineFieldType : function (element) {
		var name = "";
		$.each(Payture.options.Data, function (index, value) {
			if (Array.isArray(value)) {
				$.each(value, function () {
					if ($(this).is(element)) name = index;
				})
			} else if ($(this).is("input"))
			{
				if ($(value).is(element)) name = index;				
			}
			else if ($(this).is("select"))
				if ($(value).is(element)) name = index;
		});
		return name;
	},
	/**
	 * Get full PAN
	 *
	 * @param {HTMLElement[]} elements
	 * @return {String} pan - full PAN
	 */
	getPAN : function (elements) { return $.map(elements, function (i) { return $(i).val(); }).join(""); },
	/**
	 * Inline payment data
	 *
	 * @param {String} key - parameter Key
	 * @param {Object} cardData - card payment data
	 * @return {String} stringified payment data
	 */
	inlineData : function (key, cardData) { 
		return "Key=" + key + ";" + $.map(cardData, function (value, index) { return index + "=" + value; }).join(";"); 
	},
	winpayData : function (key, cardData) { 
		return "Key=" + key + ";" + $.map(cardData, function (value, index) { return index + "=" + value; }).join(";"); 
	},
	getFingerPrint : function (){
		$.post("/tapi/SaveUserInfo/", 
			{"data": Payture.getDataForFP()})
			.done(function (resp){$("#PaytureId").val(resp.Id);})
			.fail(function (){});
	},	
	getDataForFP : function(){
		var dateSend = new Date();
		Payture.options.TimeSubmitStart = dateSend.getTime() - Payture.options.TimeInit;
		var senddata = {};	
			if (typeof window.console !== "undefined") {
			
			Payture.options.fingerPrint.push({
					key: "first_keypress_timespan",
					value: Payture.getFirstKeypressTimespanInMs()
				}, {
					key: "total_activity_timespan",
					value: Payture.getTotalActivityAfterFirstTimespanInMs()
				},{
					key: "mouse_total_distance",
					value: Payture.getTotalDistance()
				},{
					key: "mouse_first_click_timespan",
					value: Payture.getFirstMouseClickTimespanInMs()
				});

				
				Payture.options.fingerPrint.forEach(function (item, i, keys) {
					senddata[item.key] = item.value;
				});
			}
		return JSON.stringify(senddata);
	},
	/**
	/**
	 * Inline payment data + BrainTree nonce
	 *
	 * @param {String} key - parameter Key
	 * @param {Object} cardData - card payment data
	 * @return {String} stringified payment data
	 */
	inlineDataBrainTree : function (key, cardData, attrArray) { 
		return "Key=" + key + ";" + $.map(cardData, function (value, index) { return index + "=" + value; }).join(";") + ";" + attrArray.join(";"); 
	},
	/**
	 * Send Data
	 *
	 * @param {String} action - type of action - PaySubmit or AddSubmit
	 * @param {String} data - inlined data
	 */
	submitForm : function (action, data) {
		var newForm = $("<form></form>").attr({
			method : "POST",
			action : action
		});
		newForm.appendTo($("body"));
		$("<input />").attr({
			type : "hidden",
			name : "Data"
		}).appendTo(newForm).val(data);
		newForm.submit();
	},
	/**
	 * Send Data
	 *
	 * @param {String} action
	 * @param {String} api - type of API - InPay or eWallet
	 * @param {String} data - payment data
	 * @param {Function} callbackSuccess - success callback function
	 * @param {Function} callbackError - error callback function
	 */
	submitFormJSON : function (action, api, data, callbackSuccess, callbackError) {
		$.ajax({
			type : "POST",
			url : Payture.apiPath() + "/" + action,
			data : {
				Data : data,
				Json : true
			},
			timeout : 60000,
			/**
			 * @ignore
			 * @param data
			 * @param {String} data.ErrCode
			 * @param {String} data.RedirectUrl
			 * @param {Boolean} data.Success
			 * @param {Boolean} data.CanRetry
			 * @param {String} data.Key
			 * @param {Object} data.AddInfo
			 * @param {String} data.ACSUrl
			 * @param {String} data.TermUrl
			 * @param {String} data.ThreeDSKey
			 * @param {String} data.PaReq
			 */
			success : function (data) {
				if (data.ACSUrl)
					Payture.submitForm3DS(data.ACSUrl, data.TermUrl, data.ThreeDSKey, data.PaReq);
				else if (data.Success) {
					if (typeof callbackSuccess != 'undefined' && callbackSuccess)
					{
						callbackSuccess(data.RedirectUrl);	
					}
					else{
						Payture.defaults.onSuccess(data.RedirectUrl);
					}
					$("form button").removeClass("loading").removeAttr("disabled");
				} else {
					Payture.setNewKey(data.Key);
					$("form button").removeClass("loading").removeAttr("disabled");
					if (typeof callbackError != 'undefined' && callbackError)
						callbackError(data.ErrCode, data.CanRetry, data.RedirectUrl, data.Key);
					else
						Payture.defaults.onError(data.ErrCode, data.CanRetry, data.RedirectUrl, data.Key);
				}
			},
			error : function () {
				alert(Payture.options.TemplateText[Payture.options.Language].ERRORALERT);
			}
		});
	},
	
	brainTreePay : function(data, params){
        var XHR = ("onload" in new XMLHttpRequest()) ? XMLHttpRequest : XDomainRequest;
        var xhr = new XHR();

        xhr.open('GET', '/ncapi/BrainTreeToken/', true);

        xhr.onload = function() {

            var client = new braintree.api.Client({
                clientToken: window.atob(this.responseText)
            });

            client.verify3DS({
                    amount: (parseFloat($('#amount').val())/100).toFixed(2),
                    creditCard: {
                        number: data.CardNumber,
                        expirationMonth: data.EMonth,
                        expirationYear: data.EYear,
                        cardholderName: data.CardHolder,
                        cvv: data.SecureCode
                    }
                },
                function(error, response) {
                    if (!error) {
						//$('#errorTop').hide();
						if (params.Type == "Default") Payture.submitForm("PaySubmit", Payture.inlineDataBrainTree(params.Key, params.Card, ['nonce='+response.nonce]));
						else if (params.Type == "MP") Payture.submitForm("PaySubmitMP", Payture.inlineDataBrainTree(params.Key, params.Card, ['nonce='+response.nonce]));
						else if (params.Type == "JSON") Payture.submitFormJSON("PaySubmit", params.TypeAPI, Payture.inlineDataBrainTree(params.Key, params.Card, ['nonce='+response.nonce]), params.onSuccess, params.onError);			
                    } else {
						$('#errorTop').html(error.message);//.show();
						$('form button').removeClass('loading').attr('disabled',true);
                    }
                });
        }

        xhr.onerror = function() {
            alert('Ошибка ' + this.status);
        }

        xhr.send();		
	},
	
	brainTreeAdd : function(data, params){
        var XHR = ("onload" in new XMLHttpRequest()) ? XMLHttpRequest : XDomainRequest;
        var xhr = new XHR();

        xhr.open('GET', '/ncapi/BrainTreeToken/', true);

        xhr.onload = function() {

            var client = new braintree.api.Client({
                clientToken: window.atob(this.responseText)
            });

            client.verify3DS({
                    amount: 1.01,
                    creditCard: {
                        number: data.CardNumber,
                        expirationMonth: data.EMonth,
                        expirationYear: data.EYear,
                        cardholderName: data.CardHolder,
                        cvv: data.SecureCode
                    }
                },
                function(error, response) {
                    if (!error) {
						//$('#errorTop').hide();
						if (params.Type == "Default") Payture.submitForm("AddSubmit", Payture.inlineDataBrainTree(params.Key, params.Card, ['nonce='+response.nonce,'amount=101']));
						else if (params.Type == "JSON") Payture.submitFormJSON("AddSubmit", "eWallet", Payture.inlineDataBrainTree(params.Key, params.Card, ['nonce='+response.nonce,'amount=101']), params.onSuccessAdd, params.onErrorAdd);
                    } else {
						$('#errorTop').html(error.message);//.show();
						$('form button').removeClass('loading').attr('disabled',true);
                    }
                });
        }

        xhr.onerror = function() {
            alert('Ошибка ' + this.status);
        }

        xhr.send();		
	},	
	/**
	 * Submit 3DS form
	 *
	 * @param {String} acsUrl
	 * @param {String} termUrl
	 * @param {String} md
	 * @param {String} paReq
	 */
	submitForm3DS : function (acsUrl, termUrl, md, paReq) {
		var form = $("<form name='form' action='" + acsUrl + "' method='post'>"
			 + "<input type='hidden' name='TermUrl' value='" + termUrl + "'>" 
			 + "<input type='hidden' name='MD' value='" + md + "'>" 
			 + "<input type='hidden' name='PaReq' value='" + paReq + "'>" + 
			 "</form>").appendTo($("body"));

		form.submit();
	},
	
	getFirstKeypressTimespanInMs : function(){
		
		return Payture.options.TimeFirstClick;
 
	},
	getTotalActivityAfterFirstTimespanInMs : function(){
		
		return Payture.options.TimeSubmitStart;
 
	},
	getTotalDistance : function(){
		
		return Math.round(Payture.options.TotalDistance);
 
	},
	getFirstMouseClickTimespanInMs : function(){
		
		return Payture.options.TimeMouseFirstClick;
 
	},
	
	/**
	 * Object, containing functions for field validating
	 *
	 * @namespace
	 * @static
	 * @type {Object}
	 */
	validationData : {
		CLIENT : function (cardNumber) {
			cardNumber = cardNumber.replace(/ /g,'');
			return {
				Success : ((cardNumber.length > 15 && !Payture.options.IsAmex) || (cardNumber.length > 14 && Payture.options.IsAmex)) && Payture.Moon(cardNumber),
				ErrorMessage : !Payture.Moon(cardNumber) ? Payture.options.TemplateErrors[Payture.options.Language].PanErrors:
					Payture.options.TemplateErrors[Payture.options.Language].EmptyPan,
				Element : "CardNumber"
			}
		},
		/**
		 * Validating Card Number
		 *
		 * @param {String} cardNumber
		 * @returns {validationResult}
		 */
		CardNumber : function (cardNumber) {
			cardNumber = cardNumber.replace(/ /g,'');
			return {
				Success : (((cardNumber.length > 15 && !Payture.options.IsAmex) || (cardNumber.length > 14 && Payture.options.IsAmex)) && Payture.Moon(cardNumber)),
				ErrorMessage : !Payture.Moon(cardNumber) ? Payture.options.TemplateErrors[Payture.options.Language].PanErrors:
					Payture.options.TemplateErrors[Payture.options.Language].EmptyPan,
				Element : "CardNumber"
			}
		},
		/**
		 * Validating Card Number
		 *
		 * @param {String} cardNumber
		 * @returns {validationResult}
		 */
		CardTo : function (cardNumber) {
			cardNumber = cardNumber.replace(/ /g,'');
			return {
				Success : (((cardNumber.length > 15 && !Payture.options.IsAmex) || (cardNumber.length > 14 && Payture.options.IsAmex)) && Payture.Moon(cardNumber)),
				ErrorMessage : !Payture.Moon(cardNumber) ? Payture.options.TemplateErrors[Payture.options.Language].PanErrors:
					Payture.options.TemplateErrors[Payture.options.Language].EmptyPan,
				Element : "CardTo"
			}
		},
		/**
		 * Validating Month in expire date
		 *
		 * @param {String} eMonth
		 * @returns {validationResult}
		 */
		EMonth : function (eMonth) {
			if (eMonth.toLowerCase() == "xx") {
				return {
					Success : true
				}
			}
			var eYear = Payture.getFormValues(Payture.options.Data).EYear;
			if (eMonth == "" || eMonth == "00") {
				if (eYear == "")
					return Payture.validationData.ExpiredDate(eMonth, eYear);
				return {
					Success : eMonth != "" && eMonth != "00" ,
					// ErrorMessage : eYear == "" ? Payture.options.TemplateErrors.EmptyDate : Payture.options.TemplateErrors.EmptyMonth,
					ErrorMessage : Payture.options.TemplateErrors[Payture.options.Language].EmptyMonth,
					Element : "EMonth"
				}
			} else if (eYear == "")
				return {
					Success : eMonth != "" && eMonth != "00",
					ErrorMessage : Payture.options.TemplateErrors[Payture.options.Language].EmptyMonth,
					Element : "EMonth"
				};
			else
				return Payture.validationData.ExpiredDate(eMonth, eYear);
		},
		/**
		 * Validating Year in expire date
		 *
		 * @param {String} eYear
		 * @returns {validationResult}
		 */
		EYear : function (eYear) {
			if (eYear.toLowerCase() == "xx") {
				return {
					Success : true
				}
			}
			var eMonth = Payture.getFormValues(Payture.options.Data).EMonth;
			if (eYear == "" || eYear == "00") {
				if (eMonth == "")
					return Payture.validationData.ExpiredDate(eMonth, eYear);
				return {
					Success : eYear != "" && eYear != "00",
					ErrorMessage : Payture.options.TemplateErrors[Payture.options.Language].EmptyYear,
					// ErrorMessage : eMonth == "" ? Payture.options.TemplateErrors.EmptyDate : Payture.options.TemplateErrors.EmptyYear,
					Element : "EYear"
				}
			} else if (eMonth == "")
				return Payture.validationData.EMonth(eMonth);
			else
				return Payture.validationData.ExpiredDate(eMonth, eYear);

		},
		EDate : function(eDate)
		{
			return {
				Success : eDate != "" && Payture.validationData.ExpiredDateMMYY(eDate),
				ErrorMessage : Payture.options.TemplateErrors[Payture.options.Language].EmptyDate,
				Element : "EDate"
			}			
		},
		/**
		 * Validating whole expire date
		 *
		 * @param {String} eMonth
		 * @param {String} eYear
		 * @returns {validationResult}
		 */
		ExpiredDate : function (eMonth, eYear) {
			var date = new Date(),
				today = date.getFullYear()*100 + date.getMonth() + 1,
				expDate = (2000 + eYear*1)*100 + eMonth*1;
				
			if(Payture.options.Fingerprint && 
			   eYear.length > 1 && 
			   eMonth.length > 1 && 
			   expDate >= today && 
			   eMonth*1 < 13 && 
			   $('#PaytureId').val()=='' && 
			   !Payture.options.FingerprintIsSend
			   )
			{
				Payture.options.FingerprintIsSend = true;
				Payture.getFingerPrint();
			}
			
			return {
				Success : expDate >= today && eMonth*1 < 13,
				ErrorMessage : Payture.options.TemplateErrors[Payture.options.Language].WrongDate,
				Element : eMonth*1 < 13 ? "EYear":"EMonth"
			}
		},
		ExpiredDateMMYY : function(eDate){
			var dateArray = eDate.split('/');
			var date = new Date(),
				today = date.getFullYear()*100 + date.getMonth() + 1,
				expDate = (2000 + dateArray[1]*1)*100 + dateArray[0]*1;
			if(expDate >= today && dateArray[0]*1 < 13)
			{
				$('#EMonth').val(dateArray[0]);
				$('#EYear').val(dateArray[1]);
				return true;
			}
			else
			{
				return false;
			}
		},
		/**
		 * Validating Card Holder name
		 *
		 * @param {String} cardHolder
		 * @returns {validationResult}
		 */
		CardHolder : function (cardHolder) {
			return {
				Success : cardHolder.length > 2,
				ErrorMessage : Payture.options.TemplateErrors[Payture.options.Language].EmptyCardHolder,
				Element : "CardHolder"
			}
		},
		PaytureId : function (cardHolder) {
			return {
				Success : true,
				ErrorMessage : '',
				Element : "PaytureId"
			}
		},
		/**
		 * Validating CVV
		 *
		 * @param {String} cvv
		 * @returns {validationResult}
		 */
		SecureCode : function (cvv) {
			return {
				Success : cvv.length > 2,
				ErrorMessage : Payture.options.TemplateErrors[Payture.options.Language].EmptyCVV,
				Element : "SecureCode"
			}
		},
		Email : function (email) {
			return {
				Success : typeof email == 'undefined' || email.length == 0 || email.match(/^\w+([\.+-]?\w+)*@\w+([\.-]?\w+)*(\.\w{2,4})+$/),
				ErrorMessage : Payture.options.TemplateErrors[Payture.options.Language].WrongEmail,
				Element : "Email"
			}
		},
		FIO : function (fio) {
			return {
				Success : typeof fio != 'undefined' && fio.length != 0,
				ErrorMessage : Payture.options.TemplateErrors[Payture.options.Language].WrongFIO,
				Element : "FIO"
			}
		},
		PHONE : function (phone) {
			return {
				Success : phone.match(/^(\+\d{1,2}\s)?\(?\d{3}\)?[\s.-]?\d{3}[\s.-]?\d{4}$/),
				ErrorMessage : Payture.options.TemplateErrors[Payture.options.Language].WrongPhone,
				Element : "PHONE"
			}
		},
		DESTINATION : function (pt) {
			return {
				Success : pt.length == 1,
				ErrorMessage : "error",
				Element : "DESTINATION"
			}
		},
	},
	/**
	 * Add first digit to value
	 *
	 * @param {String} number
	 * @return {String} two-digits string
	 */
	addValue : function (number) { return number.length == 1 ? "0" + number : number; },
	/**
	 * Select card type - visa or master
	 *
	 * @param {String} cardNumber
	 */
	selectCardType : function (cardNumber) {
		$(".CardType.card"+cardNumber).removeClass("visa master mastercard amex dinersclub discover unionpay jcb maestro forbrugsforeningen dankort elo");		
		/*
		$(".CardType").removeClass("visa master american mir");
		if (cardNumber != "") {
			if (cardNumber.charAt(0) == "4") {
				$(".CardType").addClass("visa");
				if (Array.isArray(Payture.options.Data.CardNumber))
					Payture.options.Data.CardNumber[3].attr("maxlength", 4);
				else
					Payture.options.Data.CardNumber.attr("maxlength", Payture.options.GroupCardNumber ? 19 : 16);
			} else {
				if (cardNumber.charAt(0) == "5" || cardNumber.charAt(0) == "6") $(".CardType").addClass("master");
				if (cardNumber.charAt(0) == "2") $(".CardType").addClass("mir");
				if (cardNumber.charAt(0) == "3") $(".CardType").addClass("american");
				if (Array.isArray(Payture.options.Data.CardNumber))
					Payture.options.Data.CardNumber[3].attr("maxlength", 7);
				else
					Payture.options.Data.CardNumber.attr("maxlength", Payture.options.GroupCardNumber ? 22 : 19);
			}
		}
		*/
	},
	/**
	 * Show errors
	 *
	 * @param {validationResult} error
	 */
	showErrors : function (error) {
		if (error === null)
			return false;
		if (Payture.options.onShowErrors !== null) {
			Payture.options.onShowErrors(error)
		} else if (!error.Success) {
			var element;
			if (Array.isArray(Payture.options.Data[error.Element]))
				element = Payture.options.Data[error.Element][0].parent();
			else
				element = Payture.options.Data[error.Element].parent();

			$(".hint").text(error.ErrorMessage).fadeIn(200);

			$(element).addClass("error").removeClass("valid");
		}else if (error.Success) {
			var element;
			if (Array.isArray(Payture.options.Data[error.Element]))
				element = Payture.options.Data[error.Element][0].parent();
			else
				element = Payture.options.Data[error.Element].parent();

			$(".hint").text(error.ErrorMessage).fadeOut(200);

			$(element).removeClass("error").addClass("valid");	
		}

		if(Payture.getFormValues(Payture.options.Data)[error.Element] == "")
		{
			$(element).removeClass("error");
		}
	},
	/**
	 * Hide errors
	 *
	 * @param {HTMLElement} element
	 */
	hideErrors : function (element) {
		if (Payture.options.onHideErrors !== null )
			Payture.options.onHideErrors(element)
		else {
			if (Array.isArray(element)) {
				$.each(element, function () {
					$(this).parent().find(".error").text("").hide();
					$(".hint").text("").hide();
					$(this).parent().find("input").removeClass("error");			
				})
			} else if ($(element).is("input")) {
				$(element).parent().find(".error").text("").hide();
				$(".hint").text("").hide();
				$(element).parent().removeClass("error");//.addClass('valid');
			}
		}
	},
	/**
	 * Set button state - disable or enable
	 */
	checkButton : function () {
		var isValid = true;
		$.each(Payture.getFormValues(Payture.options.Data), function (index, value) {
			if (index == "CardId" || index == "AddCard") return true;
			if (!Payture.Validate({ Name : index, Value : value }).Success) isValid = false;
		});
		//if(!$("#agree").prop('checked')) isValid = false;
		$("form button[name=pay]:visible").attr("disabled", !isValid);
	},
	setComission : function(){
		var amount = parseFloat($('#Amount').val());
		var comission = 0;

		if(amount < 4091){
			comission = 45;
		}
		else{
			comission = amount/100*1.1;
		}
		Payture.options.TerminalName = 'Payturep2pPSB';

		$('#Commission').val('+'+comission.toFixed(2)+' руб. комиссия за перевод');
		$('#ResultAmount').val((amount + comission).toFixed(2));
		$('#Amount').val(amount);
		if($('#Amount').val()!='')
			Payture.replaceSelection(document.getElementById('Amount'), ' руб.');
	},
	replaceSelection : function(field, word) {
		var from = field.selectionStart, to = field.selectionEnd;
		field.value = field.value.slice(0, from) + word +
					  field.value.slice(to);
		
		field.selectionStart = field.selectionEnd =
		  from;// + word.length;
	},
	setComissionHide : function(){
		$('#Commission, #Amount').val('');
	},
	/**
	 * Separating card number with space
	 *
	 * @param {Event} event
	 * @param {HTMLInputElement} element
	 */
	separatingNumber : function (event, element) {
		/*
		event = (event) ? event : window.event;
		var charCode = (event.which) ? event.which : event.keyCode;
		var pos = Payture.getCaretPosition(element),
			val = element.value.replace(/\s/g, ""),
			length = element.value.length;
		
		if (pos % 5 == 0 && charCode != 8) 
			pos += pos < length ? 1 : 2;
		if (pos > 3 && pos % 5 == 0 && charCode == 8 && pos < 16) 
			pos += -1;

		if (val.length < 16)
			element.value = val.replace(/(\d{4})/gi,"$1 ").trim();
		else
			element.value = val.replace(/(\d{4})(\d{4})(\d{4})(\d{4,})/gi,"$1 $2 $3 $4").trim();

		Payture.setCaretPosition(element, pos);
		*/
	},
	/**
	 * Set caret position in input
	 *
	 * @param {HTMLInputElement} element
	 * @param {Number} caretPos - caret position
	 */
	setCaretPosition : function (element, caretPos) {
		/*
		if(element != null) {
			if(element.createTextRange) {
				var range = element.createTextRange();
				range.move('character', caretPos);
				range.select();
			} else {
				if (element.selectionStart) {
					element.focus();
					element.setSelectionRange(caretPos, caretPos);
				} else
					element.focus();
			}
		}
		*/
	},
	/**
	 * Get caret position in input
	 *
	 * @param {HTMLInputElement} element
	 */
	getCaretPosition : function (element) {
		/*
		var CaretPos = 0;
		if (document.selection) {
			element.focus();
			var Sel = document.selection.createRange ();
			Sel.moveStart ('character', -ctrl.value.length);
			CaretPos = Sel.text.length;
		} else if (element.selectionStart || element.selectionStart == '0')
			CaretPos = element.selectionStart;

		return (CaretPos);
		*/
	},
	/**
	 * Change focus input
	 *
	 * @param {Event} event
	 * @param {HTMLInputElement} element
	 */
	changeCaretPosition : function (event, element) {
		event = (event) ? event : window.event;
		var charCode = (event.which) ? event.which : event.keyCode,
			name = $(element).attr("name"),
			maxLength = $(element).attr("maxLength"),
			next = $(element).attr("next"),
			prev = $(element).attr("prev"),
			value = $(element).val();
			
		if ((charCode != 9) && (charCode != 37) && (charCode != 39) && (charCode != 16)) {
			if ((name == 'DateTo' && value.replace(/_/g).length == 5) || (name != 'DateTo' && value.length == maxLength && next))
				$("[name=" + next + "]").focus();

			if (charCode == 8 && (value.length == 0 || value == '__/__') && prev)
				$("[name=" + prev + "]").focus();
		}       
	},
	/**
	 * Resize card number input
	 */
	resizeInput : function () {
		/*
		var input = $("input[name='CardNumber3']");
		if (input.length == 0)
			return false;
		input.width(input.val().length > 4 ? "66px" : "57px");
		*/
	},
	/**
	 * Checked char - digital or not
	 *
	 * @param {Event} event
	 * @param {String} type - type of regexp
	 * @return {Boolean} result
	 */
	checkChar : function (event, type) {
		var charCode = (typeof event.which == "number") ? event.which : event.keyCode;
		return Payture.regexp[type](charCode);
	},
	checkString : function (string, event, type) {
		var charCode = (typeof event.which == "number") ? event.which : event.keyCode;
		return Payture.regexp[type](charCode, string);
	},	
	/**
	 * Get latin character by russian character
	 *
	 * @param {String} str - russian or latin character
	 * @return {String} latin character
	 */
	getRuByEn : function (str) { return Payture.enToRu[str] ? Payture.enToRu[str] : str; },
	/**
	 * Transliterate string
	 *
	 * @param {String} str - russian or latin string
	 * @return {String} newStr - latin string
	 */
	changeRuToEn : function (str) {
		var newStr = "";
		$.each(str.split(""), function () { newStr += Payture.getRuByEn(this); });
		return newStr;
	},
	/**
	 * Translit string from russian to latin
	 *
	 * @param {Event} event
	 * @param {HTMLInputElement} element
	 */
	translateText : function (event, element) {
		var charCode = event.charCode ? event.charCode : event.keyCode,
			charCodeStr = String.fromCharCode(charCode),
			ruToEn = Payture.changeRuToEn(charCodeStr.toLowerCase()),
			val = element.value,
			r_simple = /[A-Z ]/i;

		if (charCode == 13)
			return;

		var cursorPos = 0;
		if (document.selection) {
			var range = document.selection.createRange();
			range.moveStart('textedit', -1);
			cursorPos = range.text.length;
		} else
			cursorPos = element.selectionStart;

		if (charCode == 8 || charCode == 9 || charCode == 32 || charCode == 46 || (charCode > 36 && charCode < 41))
			return true;
		else {
			var insertText = "";

			if (r_simple.test(String.fromCharCode(charCode))) {
				insertText = String.fromCharCode(charCode).toUpperCase();
				element.value = val.substring(0, cursorPos) + insertText + val.substring(cursorPos, val.length);
				element.selectionStart = cursorPos + 1;
				element.selectionEnd = cursorPos + 1;
			} else if (r_simple.test(ruToEn)) {
				insertText = ruToEn.toUpperCase();
				element.value = val.substring(0, cursorPos) + insertText + val.substring(cursorPos, val.length);
				element.selectionStart = cursorPos + 1;
				element.selectionEnd = cursorPos + 1;
			}
		}
		return false;
	},
	/**
	 * Convert XML to String
	 *
	 * @param {XML} xmlData
	 * @return {String} string
	 */
	xmlToString : function (xmlData) { return window.ActiveXObject ? xmlData.xml : (new XMLSerializer()).serializeToString(xmlData); },
	/**
	 * Shows mask stubs instead of inputs
	 *
	 * @param {String} cardMask
	 * @param {Boolean} showCVV
	 */
	showCardStub : function(cardMask, showCVV) {
		
		if (Array.isArray(Payture.options.Data.CardNumber)) {
			$.each(Payture.options.Data.CardNumber, function (index, value) {
				$(value).val(cardMask.substring(index*4, index*4 + 4)).attr("disabled", true);
			})
		} else if ($(Payture.options.Data.CardNumber).is("input")) {
			var groupMask = cardMask.replace(/((\d|\D){4})/gi,"$1 ").trim();
			$(Payture.options.Data.CardNumber).val(groupMask).attr("disabled", true);
		}
		
		if(!$(Payture.options.Data.EMonth).hasClass("winpay"))
		{
			if($('#date:visible').length > 0){
				$('#date:visible').val('XX/XX').attr("disabled", true);
				$(Payture.options.Data.EMonth).val("XX");
				$(Payture.options.Data.EYear).val("XX");
			}
			else{
				if ($(Payture.options.Data.EMonth).is("input"))
					$(Payture.options.Data.EMonth).val("XX").attr("disabled", true);
				if ($(Payture.options.Data.EYear).is("input"))
					$(Payture.options.Data.EYear).val("XX").attr("disabled", true);			
			}			
		}

		if ($(Payture.options.Data.CardHolder).is("input"))
			$(Payture.options.Data.CardHolder).val($('[name=CardId]').find(':selected').attr('data-cardholder'));
		
		if(!$(Payture.options.Data.CardHolder).hasClass("winpay")){
			$(Payture.options.Data.CardHolder).attr("disabled", true);
		}
		
		if ($(Payture.options.Data.SecureCode).is("input") && !$(Payture.options.Data.SecureCode).hasClass("winpay")){
			if(showCVV == true)
				$(Payture.options.Data.SecureCode).val("");
			else
				$(Payture.options.Data.SecureCode).val("XXX").attr("disabled", true);
			$(Payture.options.Data.SecureCode).focus();
		}
	},
	/**
	 * Formatting int
	 * @param {Number} number
	 * @return {String} formatted number
	 */
	formatInt : function (number) {
		if (number == null || number == undefined || number == '')
			return "0.00";

		var str = number.toString();
		str = str.replace(/(\d+)(\d\d)/g, '$1.$2');
		var point = /(\d+)(\.\d{1,2})/g.exec(str) ? /(\d+)(\.\d{1,2})/g.exec(str)[2] : "";
		str = /(\d+)(\.\d{1,2})/g.exec(str) ? /(\d+)(\.\d{1,2})/g.exec(str)[1] : str;
		str = str.replace(/(\d)(?=(\d\d\d)+(?!\d))/g, '$1&nbsp;');
		return str + point;
	},
	/**
	 * Bind MasterPass scripts
	 */
	MasterPass : function() {
		Payture.GetCardInfo();

		if (window.location.href.indexOf("checkout_resource_url") != -1 && window.location.href.indexOf("oauth_verifier") != -1)
			Payture.setMasterPass();
		else {
			$("form .payBlock").addClass("MPPayBlock");
			$("body").delegate(".masterpass", "click", function(){
				Payture.RedirectMasterPass();
				return false;
			});
		}
	},
	/**
	 * Get Card info via oauth info from MP redirect link
	 */
	GetCardInfo: function () {
		var oauth_Token = Payture.getURLParam("oauth_token");
		var oauth_Verifier = Payture.getURLParam("oauth_verifier");
		var checkout_Resource_Url = Payture.getURLParam("checkout_resource_url");

		if (oauth_Token != "" && oauth_Verifier != "" && checkout_Resource_Url != "") {
			$.ajax({
				type: "POST",
				url: "/mpapi/GetCardInfo",
				data: {
					"json": "true",
					"oauth_Token": oauth_Token,
					"oauth_Verifier": oauth_Verifier,
					"checkout_Resource_Url": checkout_Resource_Url
				},
				timeout : 60000,
				/**
				 * @ignore
				 * @param {Object} data
				 * @param {String} data.PANMask
				 */
				success: function (data) {
					if (data.PANMask != undefined) {
						Payture.showCardStub(data.PANMask, Payture.options.MasterPassUseCVV);
						
					}
				},
				error: function () {
					alert(Payture.options.TemplateText[Payture.options.Language].ERRORALERT);
				}
			});
		}
	},
	/**
	 * Set MasterPass fields after redirect
	 */
	setMasterPass : function () {
		Payture.options.MasterPassRedirected = true;
		var oauth = Payture.getURLParam("oauth_token");
		$("input[name=oauth_Token]").val(oauth);
		Payture.showCardStub("XXXXXXXXXXXXXXXX", Payture.options.MasterPassUseCVV);
		Payture.checkButton();
		$('.container-masterpass').hide(); 
		if(!Payture.options.MasterPassUseCVV){
			$('.wrapper').hide();
			setTimeout(function(){
				Payture.Pay({
					Type: "MP",
					Key: $("input[name='Key']").val(),
					Card: Payture.getFormValues({
						SecureCode : $("input[name='SecureCode']:visible"),
						oauth_Token: $("input[name='oauth_Token']"),
						amount: $("input[name='amount']"),
					})
				});			
			}, 1500);
		}
	},
	/**
	 * Redirecting to MasterPass Interface
	 */
	RedirectMasterPass : function() {
		var Amount = $("input[name='Amount']").val();
		var Product = $("input[name='Product']").val();
		if(!Product || typeof Product == 'undefined' || Product == '') {
			Product = 'Product';
		}
		$.ajax({
			type : "POST",
			url : "/mpapi/RedirectMasterPass",
			data : {
				"callbackUrl": window.location.href.replace("&", "[a]"),
				"json": "true",
				"Key": $("input[name='Key']").val(),
				"shopingCart": Amount + ":"+encodeURIComponent(Product)+"|1|" + Amount + ";"
				//the format is: totalamount:p1_name| p1_quantity| p1_amount; p2_name| p2_quantity| p2_amount;..
			},
			timeout : 60000,
			/**
			 * @ignore
			 * @param {Object} data
			 * @param {String} data.Url
			 */
			success : function (data) {
				location.href = data.Url;
			},
			error : function () {
				alert(Payture.options.TemplateText[Payture.options.Language].ERRORALERT);
			}
		});
	},
	/**
	 * Format query params from object
	 */
	formatQueryParams(data) {
		var createMapper = function(source, structureResolver) {
			if (!structureResolver) 
				structureResolver = function(item) {return item.toString();}

			return function(item) {
				switch(typeof source[item]) {
					case 'function':
					case 'object':
						return item + '=' + encodeURIComponent(structureResolver(source[item]));
					default:
						return item + '=' + encodeURIComponent(source[item].toString());	
				}
			}
		}

		var externalMapper = createMapper(data, function(structure) {
			var internalMapper = createMapper(structure);
			return Object.keys(structure).map(internalMapper).join(';');
		});

		return Object.keys(data).map(externalMapper).join('&');
	},
	/**
	 * Get parameter value from URL string by name
	 *
	 * @param {String} name
	 * @returns {String}l
	 */
	getURLParam : function (name) {
		name = name.replace(/[\[]/, "\\[").replace(/[\]]/, "\\]");
		var regex = new RegExp("[\\?&]" + name + "=([^&#]*)"),
			results = regex.exec(location.search);
		return results === null ? "" : decodeURIComponent(results[1].replace(/\+/g, " "));
	},
	showCheck : function() {
		var sumInKopeks = $('#amount_kopecs').text();
		if(sumInKopeks != ''){
			var currency = $('#amount_currency').text();
			var currencyText = "";
			if(currency == "RUR" || currency == "RUB"){
				currencyText = "руб."
			}
			var sumToShow = Payture.formatInt(sumInKopeks) + " " + currencyText;
			$("#sumFinal").html(sumToShow);

			$("#sendCheckEmail").click(function(){
				Payture.sendCheckEmail();
			})

		} else {
			$('.container.returnCheck').hide();
			$('.container.return').show();
		}
	},
	Moon:function(card_number) {
		if(!Payture.options.CheckLuna){
			return true;
		}
		else{
		  card_number = card_number.toString().replace(/\s/gi,'');
		  if (card_number.length % 2 != 0) card_number="0" + card_number;
		  var K, Sum=0;
		  for (var i=0; i<card_number.length; i+=2) Sum+=((K=card_number.charAt(i)*2)>9?K-9:K)+Number(card_number.charAt(i+1));
		  return Sum%10==0;
		}
	},
	ApplePayInit : function(){
		if (window.ApplePaySession) {
			var amount = parseInt($('input[name=Amount]').val())/100;
			var total = $('input[name=Total]').val();
			var product = $('input[name=Product]').val();
			var merchantIdentifier = 'merchant.com.payture.applepay';
			var promise = ApplePaySession.canMakePaymentsWithActiveCard(merchantIdentifier);
			promise.then(function (canMakePayments) {
				if (canMakePayments)
				{
					$('#apple-pay-button').css({'display':'inline-flex'});
					document.getElementById("apple-pay-button").onclick = function(evt) {		
						var request = {
							countryCode: 'RU',
							currencyCode: 'RUB',
							supportedNetworks: ['visa', 'masterCard'],
							merchantCapabilities: ['supports3DS'],
							total: { label: product, amount: amount },
						}		

						var session = new ApplePaySession(1, request);

						session.onvalidatemerchant = function (event) {
							var promise = Payture.performValidation(event.validationURL);
							promise.then(function (merchantSession) {
								session.completeMerchantValidation(merchantSession);
							}); 
						}
						
						session.onpaymentmethodselected = function(event) {				
							var newTotal = { type: 'final', label: product, amount: amount };
							var newLineItems =[{type: 'final',label: product, amount: amount }];
							
							session.completePaymentMethodSelection( newTotal, newLineItems );
						}
						
						session.onpaymentauthorized = function (event) {
							var promise = Payture.sendPaymentToken(event.payment.token);
							promise.then(function (data) {
								var status;
								if (data.Success){
									status = ApplePaySession.STATUS_SUCCESS;
									session.completePayment(status);
									Payture.paySuccess(data.RedirectUrl);
								} else {
									status = ApplePaySession.STATUS_FAILURE;
									session.completePayment(status);
									$('input[name=Key]').val(data.Key);
									Payture.payError(data.ErrCode, data.CanRetry, data.RedirectUrl);
								}
								
							});
						}
													
						session.oncancel = function(event) {
							
						}
						
						session.begin();
					}						
				}
				else
				{
					$('#apple-pay-button').css({'display':'none'});
				}
			}); 
		}		
	},
	paySuccess : function(url) {
		Payture.options.onSuccess(url);
	},
	payError : function(code, canRetry, url) {
		Payture.options.onError(code, canRetry, url);
	},
	performValidation : function(valURL) {
		return new Promise(function(resolve, reject) {
			$.post("/api/ApplePayValidateMerchant", { Url: valURL, IsCer: false }, function( data ) {
				resolve(data);
			});
		});
	},
	sendPaymentToken : function(paymentToken) {
		var json = JSON.stringify(paymentToken.paymentData);

		var b64string = '';
		if(window.btoa) {
			b64string = window.btoa(unescape(encodeURIComponent(json)));
		} else {
			b64string = base64.encode(unescape(encodeURIComponent(json)));
		}

		return new Promise(function(resolve, reject) {
			var xhr = new XMLHttpRequest();
			xhr.onload = function() {
				var data = JSON.parse(this.responseText);
				resolve(data);
			};
			xhr.onerror = reject;

			var queryData = { Json: true, PayToken: b64string, Data: { Key: $('input[name=Key]').val() }	}

			if (Payture.options.TypeAPI == 'eWallet') {
				queryData.AddCard = $("input[name='AddCard']:visible").length ? $("input[name='AddCard']:visible").is(":checked") : false;
			}

			xhr.open('POST', Payture.apiPath() + '/PaySubmitMobile?' + Payture.formatQueryParams(queryData), true);
			xhr.send();
		});
	},
	
	AndroidPayInit : function(){
		if (!window.ApplePaySession && window.PaymentRequest) {
			var request = Payture.AndroidPaymentRequest();

			request.canMakePayment()
					.then(function(result) {
						if (result) {
							$('#android-pay-button').css({'display':'block'});
							document.querySelector('#android-pay-button').addEventListener('click', function(){Payture.AndroidPayProceed();});						
						}
					})
					.catch(function(err) {
						
					});
		}
	},
	
	AndroidPaymentRequest: function(){

		var amount = parseInt($('input[name=Amount]').val())/100;
		var product = $('input[name=Product]').val();

		var supportedInstruments = [
			{
				supportedMethods: ['https://android.com/pay'],
				data: {
					merchantId: '11218867777662038582',
					allowedCardNetworks: ['MASTERCARD', 'VISA'],
					paymentMethodTokenizationParameters: {
						tokenizationType: 'NETWORK_TOKEN',
						parameters: {
						'publicKey': 'BHH5N7sxc8VnjIKC8OX5RuQyoIO7XfSGfZwudLkMmTMhy/GiDjwA6EJddJ78k/uejgZNNvQeXjUl7GeSIxeCJiI='
						}
					}
				}
			}
		];
		
		var details = {
			displayItems: [{
			label: product,
			amount: { currency: 'RUB', value: amount }
			}],
			total: {
			label: product,
			amount: { currency: 'RUB', value : amount }
			}
		};

		var options = {
			requestPayerEmail: true,
			requestPayerPhone: true,
			requestPayerName: true
		};

		return new PaymentRequest(supportedInstruments, details, options);		
	},
	
	AndroidPayProceed : function(){
		var ANDROID_PAY = 'https://android.com/pay';
		var amount = parseInt($('input[name=Amount]').val())/100;
		var product = $('input[name=Product]').val();
		
		var supportedInstruments = [
			/*{
				supportedMethods: ['basic-card'],
				data: {
					supportedNetworks: ['mastercard','visa'],
				}
			},*/
			{
				supportedMethods: ['https://android.com/pay'],
				data: {
					merchantId: '11218867777662038582',
					allowedCardNetworks: ['MASTERCARD', 'VISA'],
					paymentMethodTokenizationParameters: {
						tokenizationType: 'NETWORK_TOKEN',
						parameters: {
						'publicKey': 'BHH5N7sxc8VnjIKC8OX5RuQyoIO7XfSGfZwudLkMmTMhy/GiDjwA6EJddJ78k/uejgZNNvQeXjUl7GeSIxeCJiI='
						}
					}
				}
			}
		];
		
		var details = {
			displayItems: [{
			label: product,
			amount: { currency: 'RUB', value: amount }
			}],
			total: {
			label: product,
			amount: { currency: 'RUB', value : amount }
			}
		};

		var options = {
			requestPayerEmail: true,
			requestPayerPhone: true,
			requestPayerName: true
		};

		// Initialization
		var payment = new PaymentRequest(supportedInstruments, details, options);

		// Show UI then continue with user payment info
		payment.show().then(function(result){
			var b64string = '';
			if(window.btoa) {
				b64string = window.btoa(unescape(encodeURIComponent(result.details.paymentMethodToken)));
			} else {
				b64string = base64.encode(unescape(encodeURIComponent(result.details.paymentMethodToken)));
			}

			var queryData = { Amount: amount, Json: true, PayToken: b64string,	Data: { Key: $('input[name=Key]').val() }	}

			if (Payture.options.TypeAPI == 'eWallet') {
				queryData.AddCard = $("input[name='AddCard']:visible").length ? $("input[name='AddCard']:visible").is(":checked") : false;
			}

			return fetch(Payture.apiPath() + '/PaySubmitMobile', {
			method: 'POST',
			credentials: 'include',
		    headers: {  
				"Content-type": "application/x-www-form-urlencoded; charset=UTF-8"  
			},  
			body: Payture.formatQueryParams(queryData),
			}).then(function(res){
			if (res.status === 200) {
				return res.json();
			} else {
				throw 'Failure';
			}
		}).then(function(response){
			if (response.Success == true) {
				Payture.paySuccess(response.RedirectUrl);
				return result.complete('success');
			} else {
				Payture.payError(response.ErrCode, response.CanRetry, response.RedirectUrl);
				return result.complete('fail');
			}
		}).then(function(){
			console.log('Thank you!');
		}).catch(function(){
			return result.complete('fail');
		});
		}).catch(function(err) {
			console.log(err);
		});
	},
	
	GooglePayInit:function(){
		if (window.PaymentRequest) {
				document.querySelector('#google-pay-button').style.display = 'block';
				document.querySelector('#google-pay-button').addEventListener('click', function(){Payture.GooglePayProceed();});						
		}
	},
	
	GoogleWebPayInit:function(){
		if (window.PaymentRequest) {
			onGooglePayLoaded();			
		}
	},	
	
	GooglePayProceed:function(){
		
		var amount = parseInt($('input[name=Amount]').val())/100;
		var product = $('input[name=Product]').val();
		
		var supportedInstruments = [
			{
				supportedMethods: ['basic-card'],
				data: {
					supportedNetworks: ['mastercard','visa'],
				}
			}
		];
		
		var details = {
			displayItems: [{
			label: product,
			amount: { currency: 'RUB', value: amount }
			}],
			total: {
			label: product,
			amount: { currency: 'RUB', value : amount }
			}
		};

		var options = {
			requestPayerEmail: true,
			requestPayerPhone: true,
			requestPayerName: true
		};

		// Initialization
		var payment = new PaymentRequest(supportedInstruments, details, options);

		// Show UI then continue with user payment info
		payment.show().then(function(result){
			
			var data;
			if ('paymentMethodToken' in result.details) {
				data = result.details.paymentMethodToken
			}else{
				data = JSON.stringify(result);
			}
			
			var b64string = '';
			if(window.btoa) {
				b64string = window.btoa(unescape(encodeURIComponent(data)));
			} else {
				b64string = base64.encode(unescape(encodeURIComponent(data)));
			}

			var queryData = { Amount: amount, Json: true, PayToken: b64string,	Data: { Key: $('input[name=Key]').val() }	}

			if (Payture.options.TypeAPI == 'eWallet') {
				queryData.AddCard = $("input[name='AddCard']:visible").length ? $("input[name='AddCard']:visible").is(":checked") : false;
			}

			return fetch(Payture.apiPath() + '/PaySubmitMobile', {
				method: 'POST',
				credentials: 'include',
				headers: {  
					"Content-type": "application/x-www-form-urlencoded; charset=UTF-8"  
				},  
				body: Payture.formatQueryParams(queryData)				
			})
			.then(function(res){	
				if (res.status === 200) {
					return res.json();
				} else {
					throw 'Failure';
				}
			})
		.then(function(response){
			if (response.ACSUrl)
				Payture.submitForm3DS(response.ACSUrl, response.TermUrl, response.ThreeDSKey, response.PaReq);
			else if (response.Success) {
				if (typeof callbackSuccess != 'undefined' && callbackSuccess)
				{
					callbackSuccess(data.RedirectUrl);	
				}
				else{
					Payture.defaults.onSuccess(response.RedirectUrl);
				}
				$("form button").removeClass("loading").removeAttr("disabled");
			} else {
				Payture.setNewKey(response.Key);
				$("form button").removeClass("loading").removeAttr("disabled");
				if (typeof callbackError != 'undefined' && callbackError)
					callbackError(response.ErrCode, response.CanRetry, response.RedirectUrl, response.Key);
				else
					Payture.defaults.onError(response.ErrCode, response.CanRetry, response.RedirectUrl, response.Key);
			}
		}).then(function(){
			console.log('Thank you!');
		}).catch(function(){
			return result.complete('fail');
		});
		}).catch(function(err) {
			console.log(err);
		});
	},
	SamsungPayInit:function(){
		document.querySelector('#samsung-pay-button').style.display = 'block';
		document.querySelector('#samsung-pay-button').addEventListener('click', function(){Payture.SamsungPayProceed();});
	},
	SamsungPayProceed: function() {
		var data = {"callback":"https://secure.payture.com/Templates/SamsungPay/success.html?Key="+encodeURIComponent('Key='+$('input[name=Key]').val()), "paymentDetails":{"service":{"id":"{serviceId}"}, "orderNumber":Payture.getURLParam("SessionId"), "recurring":false, "protocol":{"type":"3DS","version":"80"}, "amount":{"option":"FORMAT_TOTAL_ESTIMATED_AMOUNT","currency":"RUB","total":parseInt($('input[name=Amount]').val())/100}, "merchant":{"name":"Payture", "reference":"123456789"}, "allowedBrands":["VI", "MC"]}};
		$.ajax({
			url: '/peapi/SamsungPayTransactionsPOST',
			type: 'post',
			data: JSON.stringify(data),
			headers: {
				"Content-type": "application/json",
				"X-Request-Id": new Date().getTime()  
			},
			dataType: 'json',
			success: function (response) {
				SamsungPay.connect(
					response.id, 
					response.href, 
					Payture.options.serviceId, 
					'https://secure.payture.com/Templates/SamsungPay/success.html',
					'https://secure.payture.com/Templates/SamsungPay/error.html',
					'ru', 
					response.encInfo.mod, 
					response.encInfo.exp, 
					response.encInfo.keyId
				);				
			}
		});	
	}
};

var QueryString = function () {
	var query_string = {};
	var query = window.location.search.substring(1);
	var vars = query.split("&");
	for (var i=0;i<vars.length;i++) {
		var pair = vars[i].split("=");
		if (typeof query_string[pair[0]] === "undefined") {
			query_string[pair[0]] = decodeURIComponent(pair[1]);
		} else if (typeof query_string[pair[0]] === "string") {
			var arr = [ query_string[pair[0]],decodeURIComponent(pair[1]) ];
			query_string[pair[0]] = arr;
		} else {
			query_string[pair[0]].push(decodeURIComponent(pair[1]));
		}
	} 
	return query_string;
}();

function reject(){}
function pf(el){$('.placeholder', $(el).parent()).hide();}
function pb(el){if($(el).val()=='')$('.placeholder', $(el).parent()).show();}
function nf(el){$(el).parent().addClass('active');}
function nb(el){$(el).parent().removeClass('active');}	


;(function(root) {		var freeExports = typeof exports == 'object' && exports;	var freeModule = typeof module == 'object' && module &&		module.exports == freeExports && module;	var freeGlobal = typeof global == 'object' && global;	if (freeGlobal.global === freeGlobal || freeGlobal.window === freeGlobal) {		root = freeGlobal;	}	var InvalidCharacterError = function(message) {		this.message = message;	};	InvalidCharacterError.prototype = new Error;	InvalidCharacterError.prototype.name = 'InvalidCharacterError';	var error = function(message) {		throw new InvalidCharacterError(message);	};	var TABLE = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/';	var REGEX_SPACE_CHARACTERS = /<%= spaceCharacters %>/g;	var decode = function(input) {		input = String(input)			.replace(REGEX_SPACE_CHARACTERS, '');		var length = input.length;		if (length % 4 == 0) {			input = input.replace(/==?$/, '');			length = input.length;		}		if (			length % 4 == 1 ||			/[^+a-zA-Z0-9/]/.test(input)		) {			error(				'Invalid character: the string to be decoded is not correctly encoded.'			);		}		var bitCounter = 0;		var bitStorage;		var buffer;		var output = '';		var position = -1;		while (++position < length) {			buffer = TABLE.indexOf(input.charAt(position));			bitStorage = bitCounter % 4 ? bitStorage * 64 + buffer : buffer;			if (bitCounter++ % 4) {				output += String.fromCharCode(					0xFF & bitStorage >> (-2 * bitCounter & 6)				);			}		}		return output;	};	var encode = function(input) {		input = String(input);		if (/[^\0-\xFF]/.test(input)) {			error(				'The string to be encoded contains characters outside of the ' +				'Latin1 range.'			);		}		var padding = input.length % 3;		var output = '';		var position = -1;		var a;		var b;		var c;		var d;		var buffer;			var length = input.length - padding;		while (++position < length) {			a = input.charCodeAt(position) << 16;			b = input.charCodeAt(++position) << 8;			c = input.charCodeAt(++position);			buffer = a + b + c;			output += (				TABLE.charAt(buffer >> 18 & 0x3F) +				TABLE.charAt(buffer >> 12 & 0x3F) +				TABLE.charAt(buffer >> 6 & 0x3F) +				TABLE.charAt(buffer & 0x3F)			);		}		if (padding == 2) {			a = input.charCodeAt(position) << 8;			b = input.charCodeAt(++position);			buffer = a + b;			output += (				TABLE.charAt(buffer >> 10) +				TABLE.charAt((buffer >> 4) & 0x3F) +				TABLE.charAt((buffer << 2) & 0x3F) +				'='			);		} else if (padding == 1) {			buffer = input.charCodeAt(position);			output += (				TABLE.charAt(buffer >> 2) +				TABLE.charAt((buffer << 4) & 0x3F) +				'=='			);		}		return output;	};	var base64 = {		'encode': encode,		'decode': decode,		'version': '<%= version %>'	};	if (		typeof define == 'function' &&		typeof define.amd == 'object' &&		define.amd	) {		define(function() {			return base64;		});	}	else if (freeExports && !freeExports.nodeType) {		if (freeModule) {			freeModule.exports = base64;		} else { 			for (var key in base64) {				base64.hasOwnProperty(key) && (freeExports[key] = base64[key]);			}		}	} else { 		root.base64 = base64;	}}(this));

/**
 * Payment methods accepted by your gateway
 *
 * @todo confirm support for both payment methods with your gateway
 */
var allowedPaymentMethods = ['TOKENIZED_CARD']; /* + CARD */

/**
 * Card networks supported by your site and your gateway
 *
 * @see {@link https://developers.google.com/pay/api/web/object-reference#CardRequirements|CardRequirements}
 * @todo confirm card networks supported by your site and gateway
 */
var allowedCardNetworks = ['AMEX', 'DISCOVER', 'JCB', 'MASTERCARD', 'VISA'];

/**
 * Identify your gateway and your site's gateway merchant identifier
 *
 * The Google Pay API response will return an encrypted payment method capable of
 * being charged by a supported gateway after shopper authorization
 *
 * @todo check with your gateway on the parameters to pass
 * @see {@link https://developers.google.com/pay/api/web/object-reference#Gateway|PaymentMethodTokenizationParameters}
 */
var tokenizationParameters = {
  tokenizationType: 'PAYMENT_GATEWAY',
  parameters: {
    'gateway': 'payture',
    'gatewayMerchantId': 'a4a6w4wzhq02i2i0pmpj1i37h8b1t4bja'
  }
}

/**
 * Initialize a Google Pay API client
 *
 * @returns {google.payments.api.PaymentsClient} Google Pay API client
 */
function getGooglePaymentsClient() {
  return (new google.payments.api.PaymentsClient({environment: 'PRODUCTION'}));
}

/**
 * Initialize Google PaymentsClient after Google-hosted JavaScript has loaded
 */
function onGooglePayLoaded() {
  var paymentsClient = getGooglePaymentsClient();
  paymentsClient.isReadyToPay({allowedPaymentMethods: allowedPaymentMethods})
      .then(function(response) {
        if (response.result) {
          addGooglePayButton();
          prefetchGooglePaymentData();
        }
      })
      .catch(function(err) {
        // show error in developer console for debugging
        console.error(err);
      });
}

/**
 * Add a Google Pay purchase button alongside an existing checkout button
 *
 * @see {@link https://developers.google.com/pay/api/brand-guidelines|Google Pay brand guidelines}
 */
function addGooglePayButton() {
  var button = document.createElement('button');
  // identify the element to apply Google Pay branding in related CSS
	button.className = 'googlepay short no-shadow';
	button.type = 'button';
  button.addEventListener('click', onGooglePaymentButtonClicked);
  document.getElementById('googlepay').appendChild(button);
}

/**
 * Configure support for the Google Pay API
 *
 * @see {@link https://developers.google.com/pay/api/web/object-reference#PaymentDataRequest|PaymentDataRequest}
 * @returns {object} PaymentDataRequest fields
 */
function getGooglePaymentDataConfiguration() {
  return {
    // @todo a merchant ID is available for a production environment after approval by Google
    // @see {@link https://developers.google.com/pay/api/web/test-and-deploy|Test and deploy}
    merchantId: '11218867777662038582',
    paymentMethodTokenizationParameters: tokenizationParameters,
    allowedPaymentMethods: allowedPaymentMethods,
    cardRequirements: {
      allowedCardNetworks: allowedCardNetworks
    }
  };
}

/**
 * Provide Google Pay API with a payment amount, currency, and amount status
 *
 * @see {@link https://developers.google.com/pay/api/web/object-reference#TransactionInfo|TransactionInfo}
 * @returns {object} transaction info, suitable for use as transactionInfo property of PaymentDataRequest
 */
function getGoogleTransactionInfo() {
	var amount = parseInt($('input[name=Amount]').val())/100;
	var product = $('input[name=Product]').val();
  return {
    currencyCode: 'RUB',
    totalPriceStatus: 'FINAL',
    // set to cart total
    totalPrice: amount

  };
}

/**
 * Prefetch payment data to improve performance
 */
function prefetchGooglePaymentData() {
  var paymentDataRequest = getGooglePaymentDataConfiguration();
  // transactionInfo must be set but does not affect cache
  paymentDataRequest.transactionInfo = {
    totalPriceStatus: 'NOT_CURRENTLY_KNOWN',
    currencyCode: 'RUB'
  };
  var paymentsClient = getGooglePaymentsClient();
  paymentsClient.prefetchPaymentData(paymentDataRequest);
}

/**
 * Show Google Pay chooser when Google Pay purchase button is clicked
 */
function onGooglePaymentButtonClicked() {
  var paymentDataRequest = getGooglePaymentDataConfiguration();
  paymentDataRequest.transactionInfo = getGoogleTransactionInfo();

  var paymentsClient = getGooglePaymentsClient();
  paymentsClient.loadPaymentData(paymentDataRequest)
      .then(function(paymentData) {
        // handle the response
        processPayment(paymentData);
      })
      .catch(function(err) {
        // show error in developer console for debugging
        console.error(err);
      });
}

/**
 * Process payment data returned by the Google Pay API
 *
 * @param {object} paymentData response from Google Pay API after shopper approves payment
 * @see {@link https://developers.google.com/pay/api/web/object-reference#PaymentData|PaymentData object reference}
 */
function processPayment(paymentData) {

	var b64string = '';
	if(window.btoa) {
		b64string = window.btoa(unescape(encodeURIComponent(paymentData.paymentMethodToken.token)));
	} else {
		b64string = base64.encode(unescape(encodeURIComponent(paymentData.paymentMethodToken.token)));
	}
	var amount = parseInt($('input[name=Amount]').val());
	var product = $('input[name=Product]').val();

	var queryData = { Amount: amount, Json: true, PayToken: b64string,	Data: { Key: $('input[name=Key]').val() }	}

	if (Payture.options.TypeAPI == 'eWallet') {
		queryData.AddCard = $("input[name='AddCard']:visible").length ? $("input[name='AddCard']:visible").is(":checked") : false;
	}
	
	$('#form, .footer-container').hide();
	return fetch(Payture.apiPath() + '/PaySubmitMobile', {
		method: 'POST',
		credentials: 'include',
			headers: {  
			"Content-type": "application/x-www-form-urlencoded; charset=UTF-8"  
		},  
		body: Payture.formatQueryParams(queryData) 
		})
		.then(function(res){
			if (res.status === 200) {
				return res.json();
			} else {
				throw 'Failure';
			}
		})
		.then(function(response){
				if (response.ACSUrl)
					Payture.submitForm3DS(response.ACSUrl, response.TermUrl, response.ThreeDSKey, response.PaReq);
				else if (response.Success) {
					Payture.paySuccess(response.RedirectUrl);
					$("form button").removeClass("loading").removeAttr("disabled");
				} else {
					Payture.setNewKey(response.Key);
					$("form button").removeClass("loading").removeAttr("disabled");
					Payture.payError(response.ErrCode, response.CanRetry, response.RedirectUrl, response.Key);
				}			
		});
}
	